-- $Header:   //ARCHIVE/Archive/DB/sql/CA7AutoDiscovery.sqv   1.25   Feb 07 2001 11:49:28   Xiaoyan  $
--
-- CA7AutoDiscovery.sql
--
-- This file defines the work tables and stored procedures necessary for doing
-- CA7 auto-discovery

USE Object
GO

---------------------------------------------------------------------------------------------------
-- add the format type and action type 
SET NOCOUNT ON 
EXEC asisp_defineFormatType 11, 'CA7', 'Events from CA7'
EXEC asisp_defineActionType 11, 1, 'STATE_CHANGE', 'State Change events from CA7'

-- add the data types
EXEC asisp_defineDataType 11, 0, 'DATE_TIME', 'Date and time of message'
EXEC asisp_defineDataType 11, 1, 'SEQ_NO', 'Sequence Number'
EXEC asisp_defineDataType 11, 46, 'MESSAGE', 'Message text'

-- add the fields
EXEC ADD_REQUIRED_DATA_TYPE 'CA7', 'STATE_CHANGE', 'DATE_TIME'
EXEC ADD_REQUIRED_DATA_TYPE 'CA7', 'STATE_CHANGE', 'SEQ_NO'
EXEC ADD_REQUIRED_DATA_TYPE 'CA7', 'STATE_CHANGE', 'MESSAGE'

-- define event procedure mapping
EXEC asisp_defineEventProcMapping 'CA7', 'STATE_CHANGE', 'Process_CA7_Event'

-- define StagedEventProc
EXEC asisp_defineStagedEventProc 'asisp_processStagedCA7Events', 1000, 1
SET NOCOUNT OFF 

---------------------------------------------------------------------------------------------------
-- create ca7_bcp table
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ca7_bcp' AND type = 'U')
BEGIN
  DROP TABLE ca7_bcp
END
GO

-- create ca7_bcp table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'ca7_bcp' AND type = 'U')
BEGIN
  PRINT 'Creating table ca7_bcp'
  EXEC('
  CREATE TABLE ca7_bcp
  (
    name_schedule_set 	ObjName PRIMARY KEY (name_schedule_set, name_job, name_system, sched_id, time_start, day_start),
    name_system 	ObjName DEFAULT NULL,
    name_job 		ObjName,
    sched_id            INT DEFAULT 0,
    flag_keyjob 	TINYINT DEFAULT 0,
    flag_exec           TINYINT DEFAULT 1,
    year_start 		INT DEFAULT 2000,
    day_start 		SMALLINT DEFAULT 0,
    time_start 		VARCHAR(20) DEFAULT NULL,
    time_duration 	VARCHAR(20) NULL,
    year_stop 		INT DEFAULT 2000,
    day_stop 		SMALLINT DEFAULT 0,
    time_stop 		VARCHAR(20) NULL
  )')
END
GO

---------------------------------------------------------------------------------------------------
-- create ca7_schedules table
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ca7_schedules' AND type = 'U')
BEGIN
  DROP TABLE ca7_schedules
END
GO

-- create table ca7_schedules
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'ca7_schedules' AND type = 'U')
BEGIN
  PRINT 'Creating table ca7_schedules'
  EXEC ('CREATE TABLE ca7_schedules
  (
    obj_id ObjID DEFAULT 0,
    name ObjName PRIMARY KEY (name, name_schedule_set, bulk_load),
    name_schedule_set ObjName DEFAULT NULL,
    bcys_id ObjID DEFAULT 0,
    bulk_load int DEFAULT 1
  )')
END
GO
---------------------------------------------------------------------------------------------------
-- create ca7_jobs table
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ca7_jobs' AND type = 'U')
BEGIN
  DROP TABLE ca7_jobs
END
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'ca7_jobs' AND type = 'U')
BEGIN
  PRINT 'Creating table ca7_jobs'
  EXEC ('CREATE TABLE ca7_jobs
  (
    obj_id 	ObjID DEFAULT 0,
    name 	ObjName PRIMARY KEY (name, cycle_name, sched_id, btcy_id),
    cycle_name 	ObjName DEFAULT NULL,
    sched_id	TINYINT DEFAULT 0,
    flag_keyjob TINYINT DEFAULT 0,
    year_start INT DEFAULT 1900,
    day_start SMALLINT DEFAULT 0,
    time_start VARCHAR(20) DEFAULT NULL,
    time_duration VARCHAR(20) NULL,
    year_stop INT DEFAULT 1900,
    day_stop SMALLINT DEFAULT 0,
    time_stop VARCHAR(20) NULL,
    start_date_time DATETIME DEFAULT ''19000101'',
    end_date_time DATETIME DEFAULT ''19000101'',
    duration INT DEFAULT 0,
    btcy_id ObjID DEFAULT 0,
    bulk_load INT DEFAULT 1
  )'
  )
END
GO

---------------------------------------------------------------------------------------------------
-- create ca7_bcys_name_lookup table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'ca7_bcys_name_lookup' AND type = 'U')
BEGIN
  PRINT 'Creating table ca7_bcys_name_lookup'
  EXEC ('CREATE TABLE ca7_bcys_name_lookup
  (short_name 	ObjName PRIMARY KEY,
   full_name	ObjName)'
  )
END
GO


---------------------------------------------------------------------------------------------------
-- create asisp_setCA7bcysnamelookup procedure
PRINT 'Creating procedure asisp_setCA7bcysnamelookup'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_setCA7bcysnamelookup')
  DROP PROCEDURE asisp_setCA7bcysnamelookup
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7AutoDiscovery.sqv  $
-- $Revision:   1.25  $
-- $Date:   Feb 07 2001 11:49:28  $
-- $Author:   Xiaoyan  $
-- set up CA7 bcys name lookup for short name to full name, short name is the name used by MVS
-- and full name is the one shown in TBSM
CREATE PROCEDURE asisp_setCA7bcysnamelookup
@short_name ObjName, 
@full_name ObjName
AS

  RAISERROR('Set bcys name lookup for short name = %s and full name = %s', 0, 1, @short_name, @full_name) 
  SET NOCOUNT ON 

  IF NOT EXISTS (SELECT * FROM ca7_bcys_name_lookup WHERE short_name = @short_name)
    INSERT ca7_bcys_name_lookup SELECT @short_name, @full_name
  ELSE 
    UPDATE ca7_bcys_name_lookup SET full_name = @full_name WHERE short_name = @short_name 

  SET NOCOUNT OFF
GO

---------------------------------------------------------------------------------------------------
-- create asisp_setCA7Globals procedure
PRINT 'Creating procedure asisp_setCA7Globals'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_setCA7Globals')
  DROP PROCEDURE asisp_setCA7Globals
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7AutoDiscovery.sqv  $
-- $Revision:   1.25  $
-- $Date:   Feb 07 2001 11:49:28  $
-- $Author:   Xiaoyan  $
-- set up global variable to handle NULL SYSTEM field in SIRD-11 evnets. For propertyname = 'keep_unknown'
-- and groupname = 'CA7', if propertyvalue = 1, create all jobs under '*UNKNOWN*' schedule as non-key, 
-- If propertyvalue = 0, discard the events.  
CREATE PROCEDURE asisp_setCA7Globals
@groupname VARCHAR(50), 
@propertyname VARCHAR(50), 
@propertyvalue VARCHAR(50), 
@propertydatatype VARCHAR(50)
AS

  RAISERROR('Set Global variable %s for %s', 0, 1, @propertyname, @groupname) 
  SET NOCOUNT ON 

  IF NOT EXISTS (SELECT * FROM SystemConfiguration 
                   WHERE groupname = @groupname and propertyname = @propertyname)
    INSERT SystemConfiguration(groupname, propertyname, propertyvalue, propertydatatype)
      SELECT @groupname, @propertyname, @propertyvalue, @propertydatatype
 
  ELSE 
    UPDATE SystemConfiguration SET groupname = @groupname, 
      propertyname = @propertyname, 
      propertyvalue = @propertyvalue, 
      propertydatatype = @propertydatatype

  SET NOCOUNT OFF
GO

------------------------------------------------------------------------------------------------
-- Create procedure asisp_setCA7bulkloadStatus
IF EXISTS(SELECT * FROM sysobjects WHERE name = 'asisp_setCA7bulkloadStatus' AND type = 'P')
  DROP PROCEDURE asisp_setCA7bulkloadStatus
GO

PRINT 'Creating procedure asisp_setCA7bulkloadStatus....'
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7AutoDiscovery.sqv  $
-- $Revision:   1.25  $
-- $Date:   Feb 07 2001 11:49:28  $
-- $Author:   Xiaoyan  $
-- pstep and pstat of Ca7 autodiscovery process are setup in the database as:
-- @pstep = 0, @pstat = 0  	ASIIPListenerSvc get ready file for the schedule set.
-- @pstep = 1, @pstat = 1  	parsing forecast file;
-- @pstep = 1, @pstat = 0 	*.bcp file ready to process;
-- @pstep = 2, @pstat = 1 	start to load *.bcp file into db;
-- @pstep = 2, @pstat = 0 	finish to load *.bcp file into db;
-- @pstep = 3, @pstat = 1 	start to process data in db;
-- entries will be deleted after process successfully
-- we will use error and errtext to atract error for each step

CREATE PROCEDURE asisp_setCA7bulkloadStatus
  @file_path VARCHAR(256),
  @pstep TINYINT = NULL,
  @pstat TINYINT = NULL,
  @batchid INT = NULL
AS
  --get FormatType from database and use pstep to define file status
  DECLARE @FormatType TINYINT, @pstep_cur TINYINT, @pstat_cur TINYINT
  SELECT @FormatType = FORMAT_TYPE FROM FORMAT_TYPE WHERE NAME = 'CA7'

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_setCA7bulkloadStatus
  ELSE
    SAVE TRAN asisp_setCA7bulkloadStatus

  IF @batchid IS NULL
    INSERT DiscoveryBatch(ctime, FormatType, FilePath, pstep, pstat)
      SELECT getdate(), @FormatType, @file_path, @pstep, @pstat
  ELSE
    UPDATE DiscoveryBatch set pstep = @pstep, pstat = @pstat
      WHERE FormatType = @FormatType AND batchid = @batchid

  IF @tcount = 0
    COMMIT TRAN asisp_setCA7bulkloadStatus
  RETURN 0

GO
------------------------------------------------------------------------------------------------
-- Create procedure _LoadCA7DiscoveryBatch
IF EXISTS(SELECT * FROM sysobjects WHERE name = '_LoadCA7DiscoveryBatch' AND type = 'P')
  DROP PROCEDURE _LoadCA7DiscoveryBatch
GO

PRINT 'Creating procedure _LoadCA7DiscoveryBatch....'
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7AutoDiscovery.sqv  $
-- $Revision:   1.25  $
-- $Date:   Feb 07 2001 11:49:28  $
-- $Author:   Xiaoyan  $
-- bulk load *.bcp files into ca7_bcp table

CREATE PROCEDURE _LoadCA7DiscoveryBatch
AS

  RAISERROR('Start to run _LoadCA7DiscoveryBatch.', 0, 1 )

  -- stardard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _LoadCA7DiscoveryBatch
  ELSE
    SAVE TRAN _LoadCA7DiscoveryBatch

  --get FormatType from database and use pstep to define file status
  DECLARE @FormatType TINYINT, @file_path VARCHAR(256), @batchid INT, @re INT
  SELECT @FormatType = FORMAT_TYPE FROM FORMAT_TYPE WHERE NAME = 'CA7'
  -- check if any ca7 autodiscovery process is running
  IF EXISTS(SELECT * FROM DiscoveryBatch WHERE FormatType = @FormatType AND pstat = 1)  
  BEGIN
    RAISERROR('Other CA7 autodiscovery process is running.', 0, 1 )
    RETURN -30000
  END

  -- cleanup for multiple entries in DiscoveryBatch. Since new forecast file is here, 
  -- we don't need old any more even the one is in step 2. How ever, we keep the entries 
  -- for old files was laoded and new one is not here yet 
  DELETE b1 FROM DiscoveryBatch AS b1 
    JOIN DiscoveryBatch AS b2 ON b1.FilePath = b2.FilePath
    AND b1.FormatType = b2.FormatType
    AND b1.batchid < b2.batchid
    WHERE b1.FormatType = @FormatType 
  
  -- update the status
  UPDATE DiscoveryBatch SET pstat = 1, pstep = 1
    WHERE FormatType = @FormatType AND pstep = 0 AND (error IS NULL OR error = 0) 

  SELECT FilePath INTO #FilePath FROM DiscoveryBatch WHERE FormatType = @FormatType AND pstep = 2
  IF @@ROWCOUNT > 0
  BEGIN
    UPDATE #FilePath SET FilePath = SUBSTRING(REVERSE(FilePath), 1, CHARINDEX('/', REVERSE(FilePath))-1)
    UPDATE #FilePath SET FilePath = REVERSE(FilePath)
    IF EXISTS (SELECT * FROM #FilePath WHERE FilePath IN (SELECT short_name FROM ca7_bcys_name_lookup))
      UPDATE f SET f.FilePath = l.full_name FROM #FilePath AS f 
        JOIN ca7_bcys_name_lookup AS l ON f.FilePath = l.short_name 
    DELETE ca7_bcp WHERE name_schedule_set NOT IN (SELECT FilePath FROM #FilePath)
  END 
  ELSE
    TRUNCATE TABLE ca7_bcp
  -- loop all the ready files in DiscoveryBatch table for CA7 even the files were loaded before. 
  DECLARE @num_schedule_set TINYINT
  SELECT @num_schedule_set = 0
  DECLARE cc CURSOR FOR SELECT batchid, FilePath FROM DiscoveryBatch 
                          WHERE FormatType = @FormatType 
                          AND (error IS NULL OR error = 0) AND pstep <> 2 AND pstat =1
  OPEN cc
  FETCH NEXT FROM cc INTO @batchid, @file_path
  WHILE @@FETCH_STATUS <> -1
  BEGIN

    DECLARE @cmd VARCHAR(1023), @xpResult INT
    SELECT @cmd = 'sh ca7.parse.ksh -d' + @file_path 

    EXEC @xpResult=master..xp_cmdshell @cmd

    IF (@xpResult != 0)
    BEGIN
      UPDATE DiscoveryBatch SET error = @@ERROR, 
		errtext = 'Failed on parse.',
                pstep = 1,
		pstat = 0
                WHERE FormatType = @FormatType AND FilePath = @file_path
      CONTINUE
    END 
    EXEC asisp_setCA7bulkloadStatus @file_path, 1, 0, @batchid
    -- bulk insert data into db
    EXEC('BULK INSERT ca7_bcp FROM ''' + @file_path + '.ca7.bcp''
    WITH (
      FIELDTERMINATOR = ''|'', 
      DATAFILETYPE = ''char'',
      FIRSTROW = 2
    )')
    IF @@ERROR <> 0
    BEGIN
      UPDATE DiscoveryBatch SET error = @@ERROR, 
		errtext = 'Failed on bulk insert.',
                pstep = 2,
		pstat = 0
                WHERE FormatType = @FormatType AND FilePath = @file_path
      CONTINUE
    END
    EXEC asisp_setCA7bulkloadStatus @file_path, 2, 0, @batchid
    SELECT @num_schedule_set = @num_schedule_set + 1 
    FETCH NEXT FROM cc INTO @batchid, @file_path
  END
  DEALLOCATE cc

  IF @tcount = 0
    COMMIT TRAN _LoadCA7DiscoveryBatch

  IF EXISTS (SELECT * FROM DiscoveryBatch WHERE FormatType = @FormatType AND error <> 0 AND pstep = 1)
  BEGIN
    RAISERROR('Please check DB error in DiscoveryBatch table', 0, 1 )
    RAISERROR('Total schedule set processed = %d', 0, 1, @num_schedule_set)
    RETURN -30000
  END
  ELSE
  BEGIN
    RAISERROR('Total schedule set processed = %d', 0, 1, @num_schedule_set)
    RETURN 0
  END

GO
---------------------------------------------------------------------------------------------------
-- Create procedure asisp_CA7createBulkBATC
PRINT 'Creating procedure asisp_CA7createBulkBATC'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_CA7createBulkBATC' AND type = 'P')
  DROP PROCEDURE asisp_CA7createBulkBATC
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7AutoDiscovery.sqv  $
-- $Revision:   1.25  $
-- $Date:   Feb 07 2001 11:49:28  $
-- $Author:   Xiaoyan  $
-- Bulk creates the BATC objects for the CA7 using ca7_bcp table entries
CREATE PROCEDURE asisp_CA7createBulkBATC
@useBCPTable INT = 1
AS

  SET NOCOUNT ON

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_CA7createBulkBATC
  ELSE
    SAVE TRAN asisp_CA7createBulkBATC

  -- create new table for new instances with IDENTITY id column seeded as (0,1) 
  CREATE TABLE #ca7_jobs_new
  (
    id ObjID IDENTITY(0,1),
    name ObjName NULL,
    sched_id INT,
    parent_cid ClassID NULL,
    parent_id ObjID
  )
  -- insert all non existing job in #ca7_jobs_new and create them in bulk
  INSERT INTO #ca7_jobs_new (name, sched_id, parent_cid, parent_id)
    SELECT DISTINCT name, sched_id, 'BTCY', btcy_id
    FROM ca7_jobs 
    WHERE obj_id = 0 AND bulk_load = @useBCPTable
  -- allocate new objects
  DECLARE @base_id ObjID, @rows INT
  SELECT @rows = COUNT(id) FROM #ca7_jobs_new
  IF @rows > 0
    EXEC @base_id = alloc_BATC @rows
  ELSE
    SET @base_id = 0
  -- insert into _C
  INSERT BATCH_C (id, name, _ParentID, _ParentCID, _SCHED_PACK)
    SELECT id+@base_id, name, parent_id, parent_cid, sched_id
    FROM #ca7_jobs_new
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN asisp_CA7createBulkBATC
    RETURN -30000
  END

  -- insert links
  INSERT INTO BTCY_BATC_PHYC (src_id, dst_id)
  SELECT parent_id, id + @base_id
  FROM #ca7_jobs_new
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRAN asisp_CA7createBulkBATC
    RETURN -30000
  END
  -- finish up
  IF @tcount = 0
    COMMIT TRAN asisp_CA7createBulkBATC

  RETURN @rows

  SET NOCOUNT OFF
GO

---------------------------------------------------------------------------------------------------
-- Create procedure _CA7CreateBATC
PRINT 'Creating procedure _CA7CreateBATC'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_CA7CreateBATC' AND type = 'P')
  DROP PROCEDURE _CA7CreateBATC
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7AutoDiscovery.sqv  $
-- $Revision:   1.25  $
-- $Date:   Feb 07 2001 11:49:28  $
-- $Author:   Xiaoyan  $
-- Creates the BATC objects for the CA7 plan as loaded in the ca7_bcp table
CREATE PROCEDURE _CA7CreateBATC
@useBCPTable INT = 1
AS

  SET NOCOUNT ON

  DECLARE @tcount INT, @rows INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _CA7CreateBATC
  ELSE
    SAVE TRAN _CA7CreateBATC

  -- fix times of 24:00 in ca7_jobs
  UPDATE ca7_jobs
  SET time_start = '00:00', day_start = day_start + 1
  WHERE time_start = '24:00'
  UPDATE ca7_jobs
  SET time_stop = '00:00', day_stop = day_stop + 1
  WHERE time_stop = '24:00'

  -- map the obj_id column to actual ids for existing objects
  -- unique jobs are based on parent, name, and _SCHED_PACK
  IF @useBCPTable = 1
  BEGIN 
    UPDATE j SET j.obj_id = b.id
      FROM ca7_jobs AS j JOIN BATCH_C AS b ON b._ParentCID = 'BTCY'
        AND b._ParentID = j.btcy_id AND j.bulk_load = @useBCPTable 
        AND b.name = j.name
        AND b._SCHED_PACK = j.sched_id 
    WHERE b.deleted = 0 
  END

  -- translate the CA7 time formats into SQL time formats
  -- the start_date_time and end_date_time variables should be defaulted to 19000101,
  -- which should work in any locale
  UPDATE ca7_jobs SET start_date_time = DATEADD(yyyy, year_start - DATEPART(yyyy, start_date_time), start_date_time)
  UPDATE ca7_jobs SET start_date_time = DATEADD(dy, day_start - 1, start_date_time)
  UPDATE ca7_jobs SET start_date_time = DATEADD(hh, DATEPART(hh, CONVERT(DATETIME,time_start)), start_date_time)
  UPDATE ca7_jobs SET start_date_time = DATEADD(mi, DATEPART(mi, CONVERT(DATETIME,time_start)), start_date_time)
  UPDATE ca7_jobs SET end_date_time = DATEADD(yyyy, year_stop - DATEPART(yyyy, end_date_time), end_date_time)
  UPDATE ca7_jobs SET end_date_time = DATEADD(dy, day_stop - 1, end_date_time)
  UPDATE ca7_jobs SET end_date_time = DATEADD(hh, DATEPART(hh, CONVERT(DATETIME,time_stop)), end_date_time)
  UPDATE ca7_jobs SET end_date_time = DATEADD(mi, DATEPART(mi, CONVERT(DATETIME,time_stop)), end_date_time)
  UPDATE ca7_jobs SET duration = CONVERT(INT, SUBSTRING(time_duration, 1, 2))*60 + CONVERT(INT, SUBSTRING(time_duration, 4, 5))

  -- bulk create batch objects using ca7_jobs table
  IF EXISTS (SELECT * FROM ca7_jobs WHERE obj_id = 0 AND bulk_load = @useBCPTable)
  BEGIN
    EXEC @rows = asisp_CA7createBulkBATC @useBCPTable
    IF @rows < 0
    BEGIN
      RAISERROR('Bulk create batch jobs failed', 0, 1) 
      ROLLBACK TRAN _CA7CreateBATC
      RETURN -30000
    END
  END
  -- update BATCH_C attributes except PriorityID which will be updated by trigger
  UPDATE b
    SET b._KeyStatus = j.flag_keyjob,
      b._SchedStartTime = j.start_date_time,
      b._SchedStopTime = j.end_date_time,
      b._SchedDuration = j.duration,
      b._SchedStatusFlags = 1
    FROM BATCH_C AS b JOIN ca7_jobs AS j ON b.name = j.name 
    AND b._SCHED_PACK = j.sched_id
    AND j.bulk_load = @useBCPTable
    JOIN link AS l ON j.btcy_id = l.src_id 
    AND l.src_cid = 'BTCY'
    AND b.id = l.dst_id 
    AND l.dst_cid = 'BATC' 
    AND l.deleted = 0

  IF @tcount = 0
    COMMIT TRAN _CA7CreateBATC
  
  IF @rows IS NULL
    RETURN 0
  ELSE
    RETURN @rows
  SET NOCOUNT OFF
GO

---------------------------------------------------------------------------------------------------
-- create update trigger on BATCH_C for _PriorityID. This is designed by CA7. 
-- Object priority for non-key job is defaulted as ignore. When key job is changed to non key
-- while priority of key job was inherit from events, the object priority will be ignore. 
-- Other priority of batch objects will not set to ignore even the job was converted from 
-- key job to non key job. 
PRINT 'Creating trigger tU_BATCH_C....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_BATCH_C' AND type = 'TR')
  DROP TRIGGER tU_BATCH_C
GO

CREATE TRIGGER tU_BATCH_C ON BATCH_C FOR UPDATE
AS
  IF UPDATE(_KeyStatus)
  BEGIN
    UPDATE b SET b._PriorityID = 5 FROM BATCH_C AS b
      JOIN inserted AS i ON b.id = i.id AND i._KeyStatus = 0 
      JOIN deleted AS d ON i.id = d.id AND d._PriorityID = 6
    UPDATE b SET b._PriorityID = 6 FROM BATCH_C AS b
      JOIN inserted AS i ON b.id = i.id AND i._KeyStatus = 1
      JOIN deleted AS d ON b.id = d.id AND d._PriorityID = 5
  END

GO

---------------------------------------------------------------------------------------------------
--_LoadCA7Plan_CreateBTCY
PRINT 'Creating procedure _LoadCA7Plan_CreateBTCY'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_LoadCA7Plan_CreateBTCY' AND type = 'P')
  DROP PROCEDURE _LoadCA7Plan_CreateBTCY
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7AutoDiscovery.sqv  $
-- $Revision:   1.25  $
-- $Date:   Feb 07 2001 11:49:28  $
-- $Author:   Xiaoyan  $
-- Creates the BTCY objects for the CA7 plan as loaded in the ca7_bcp table
CREATE PROCEDURE _LoadCA7Plan_CreateBTCY
  @name_schedule_set ObjName = NULL,
  @id_schedule_set ObjID = NULL,
  @useBCPTable TINYINT = 1
AS

  SET NOCOUNT ON

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _LoadCA7Plan_CreateBTCY
  ELSE
    SAVE TRAN _LoadCA7Plan_CreateBTCY

  IF @useBCPTable = 1
  BEGIN
    -- handle objects with empty SYSTEM field based on global variable
    INSERT INTO ca7_schedules (name, name_schedule_set, bcys_id)
      SELECT DISTINCT name_system, name_schedule_set, @id_schedule_set
      FROM ca7_bcp WHERE name_schedule_set = @name_schedule_set 
      AND name_system <> '' AND name_system IS NOT NULL

    -- map the obj_id column to actual ids
    UPDATE cyc SET cyc.obj_id = btcy.id
      FROM ca7_schedules AS cyc, BatchCycle_V AS btcy 
      JOIN containment AS c ON c.dst_cid = 'BTCY' and c.dst_id = btcy.id
      WHERE c.deleted = 0 AND cyc.name = btcy.name AND cyc.bcys_id = c.src_id AND cyc.bulk_load = @useBCPTable
  END

  -- create new instances
  CREATE TABLE #ca7_schedules_new
  (
    id ObjID IDENTITY(0,1),
    name ObjName NULL,
    parent_cid ClassID NULL,
    parent_id ObjID
  )

  INSERT INTO #ca7_schedules_new (name, parent_cid, parent_id)
  SELECT name, 'BCYS', bcys_id
  FROM ca7_schedules
  WHERE obj_id = 0 AND bulk_load = @useBCPTable

  IF EXISTS (SELECT * FROM #ca7_schedules_new)
  BEGIN
    DECLARE @base_id ObjID, @rows INT
    SELECT @rows = COUNT(id) FROM #ca7_schedules_new
    IF @rows > 0
      EXEC @base_id = alloc_BTCY @rows
    ELSE
      SET @base_id = 0

    -- insert into _C
    INSERT BatchCycle_C (id, name)
      SELECT id+@base_id, name
      FROM #ca7_schedules_new
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN _LoadCA7Plan_CreateBTCY
      RETURN -30000
    END

    -- insert into _A
    INSERT BatchCycle_A (id,_Desc,_ChildEventMatrixID,_ExceptionMatrixID,_PriorityID)
      SELECT id+@base_id, '', 0, 0, 6
      FROM #ca7_schedules_new
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN _LoadCA7Plan_CreateBTCY
      RETURN -30000
    END

    -- insert links
    INSERT INTO BCYS_BTCY_AGGC (src_id, dst_id)
      SELECT parent_id, id + @base_id
      FROM #ca7_schedules_new
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN _LoadCA7Plan_CreateBTCY
      RETURN -30000
    END
  END

  IF @tcount = 0
    COMMIT TRAN _LoadCA7Plan_CreateBTCY
  
  IF @rows IS NULL
    RETURN 0
  ELSE
    RETURN @rows
  SET NOCOUNT OFF
GO

---------------------------------------------------------------------------------------------------
-- Create procedure _DiscoverCA7Objects
PRINT 'Creating procedure _DiscoverCA7Objects'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverCA7Objects' AND type = 'P')
  DROP PROCEDURE _DiscoverCA7Objects
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7AutoDiscovery.sqv  $
-- $Revision:   1.25  $
-- $Date:   Feb 07 2001 11:49:28  $
-- $Author:   Xiaoyan  $
-- load ca7 plan from ca7_bcp table. All schedule sets have to be created before auto discovery schedule and batch job
-- objects. If schedule set was not created, raiserror will print the missing schedule set and the whole transaction will
-- roll back.  
CREATE PROCEDURE _DiscoverCA7Objects
AS
  
  RAISERROR( '_DiscoverCA7Objects', 0, 1)
  SET NOCOUNT ON

  -- Standard transaction logic  
  DECLARE @tcount INT
  SELECT @tcount = @@trancount
  IF @tcount = 0
    BEGIN TRAN _DiscoverCA7Objects
  ELSE
    SAVE TRAN _DiscoverCA7Objects

  -- Check if other process with pstep > 1 running
  DECLARE @FormatType TINYINT
  SELECT @FormatType = FORMAT_TYPE FROM FORMAT_TYPE WHERE NAME = 'CA7'
  IF EXISTS ( SELECT * FROM DiscoveryBatch WHERE FormatType = @FormatType AND pstep >1 and pstat = 1)
  BEGIN
    RAISERROR('Other loading or process is running.', 0, 1)
    RETURN -30000
  END

  -- set pstep to 3, pstat = 1 for all data ready to process 
  UPDATE DiscoveryBatch SET pstep = 3, pstat = 1 WHERE FormatType = @FormatType AND pstep =2 and pstat = 0

  -- clean up before process
  DECLARE @name_schedule_set ObjName, @id_schedule_set ObjID, @name_schedule ObjName, @id_schedule ObjID
  DECLARE @name_job ObjName, @id_job ObjID, @sched_id INT, @job_count INT, @like_clause VARCHAR(256)

  DELETE ca7_schedules WHERE bulk_load = 1
  DELETE ca7_jobs WHERE bulk_load = 1

  -- handle objects with empty SYSTEM field and non-executable jobs based on global variable.
  -- These jobs will not be created if @keep_unknown is not set to 1. 
  -- update SYSTEM = '*UNKNOWN*' for empty field if @keep_unknown = 1 
  -- or else leave it as NULL and the objects under it will not be created
  DECLARE @keep_unknown TINYINT
  SELECT @keep_unknown = CONVERT(TINYINT, propertyvalue) 
    FROM SystemConfiguration 
    WHERE propertyname = 'keep_unknown' AND groupname = 'CA7'
  IF @keep_unknown = 1
    UPDATE ca7_bcp SET name_system = '*UNKNOWN*' WHERE name_system IS NULL OR name_system = ''
  ELSE 
    DELETE ca7_bcp WHERE flag_exec = 0 AND name_system = '' AND name_system IS NULL

  -- delete multiple entries for the same job and run on different days
  DELETE c1 FROM ca7_bcp AS c1 
    JOIN ca7_bcp AS c2 ON c1.name_schedule_set = c2.name_schedule_set
    AND c1.name_system = c2.name_system
    AND c1.name_job = c2.name_job
    AND c1.sched_id = c2.sched_id
    AND (c1.day_start > c2.day_start OR c1.time_start > c2.time_start) 

  -- update schedule set short name with full name 
  UPDATE b SET b.name_schedule_set = l.full_name FROM ca7_bcp AS b
    JOIN ca7_bcys_name_lookup AS l ON b.name_schedule_set = l.short_name

  -- loop ca7_bcp table by schedule set name and created all schedules. insert all info from ca7_bcp 
  -- with _ParentCID and _ParentID for all jobs.  
  DECLARE bcp CURSOR FOR SELECT DISTINCT name_schedule_set FROM ca7_bcp
  OPEN bcp 
  FETCH NEXT FROM bcp INTO @name_schedule_set
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @like_clause = '%' + @name_schedule_set + '%' 
    -- find schedule set id, if not exists, return error
    SELECT @id_schedule_set = id FROM BatchCycles_V WHERE deleted = 0 and name = @name_schedule_set
    IF @id_schedule_set IS NULL
    BEGIN
      UPDATE DiscoveryBatch 
        SET pstep = 3, 
	    pstat = 0, 
	    error = -30000,
	    errtext = 'Schedule set was not created.'
      WHERE FormatType = @FormatType
      AND pstep = 3 AND pstat = 1 AND FilePath like @like_clause
      RAISERROR("Schedule set was not created for %s.", 0, 1, @name_schedule_set)
      CONTINUE
    END

    RAISERROR("Processing %s schedule set.", 0, 1, @name_schedule_set)
    -- create all schedules which are not exists
    DECLARE @rows int 
    EXEC @rows = _LoadCA7Plan_CreateBTCY @name_schedule_set, @id_schedule_set, 1
    IF @rows < 0
    BEGIN
      RAISERROR('Bulk create batch schedules failed', 0, 1)
      UPDATE DiscoveryBatch 
        SET pstep = 3, 
	  pstat = 0, 
	  error = -30000,
	  errtext = 'Bulk create batch schedules failed.'
        WHERE FormatType = @FormatType
        AND pstep = 3 AND pstat = 1 AND FilePath like @like_clause
      CONTINUE
    END
    -- make sure all monitored batch schedules were created under this schedule set 
    IF EXISTS (SELECT * FROM ca7_bcp 
                 WHERE name_schedule_set = @name_schedule_set 
                 AND name_system IS NOT NULL AND name_system <> ''
                 AND name_system NOT in 
                   (SELECT name FROM BatchCycle_V AS b 
                   JOIN BCYS_BTCY_AGGC AS l ON b.id = l.dst_id 
                   AND b.deleted = 0 
                   AND l.src_id = @id_schedule_set))
    BEGIN
      RAISERROR("Batch schedule %s was not created.", 0, 1, @name_schedule)
      UPDATE DiscoveryBatch 
        SET pstep = 3, 
	    pstat = 0, 
	    error = -30000,
	    errtext = 'Bulk create batch schedules failed.'
        WHERE FormatType = @FormatType
        AND pstep = 3 AND pstat = 1 AND FilePath like @like_clause
    END
    -- copy monitored jobs out of the ca7_bcp table into ca7_jobs and update btcy_id and id based on 
    -- bcys_id, system_name, job name and sched_id
    INSERT INTO ca7_jobs (
		name, 
	        sched_id,
		cycle_name, 
		flag_keyjob, 
		year_start, 
		day_start, 
		time_start, 
		time_duration, 
		year_stop, 
		day_stop, 
		time_stop,
		btcy_id,
        	bulk_load)
            SELECT c.name_job,
	        c.sched_id,
		c.name_system,
		c.flag_keyjob,
		c.year_start,
		c.day_start,
		c.time_start, 
		c.time_duration, 
		c.year_stop, 
		c.day_stop, 
		c.time_stop,
		b.id,
        	1
      FROM ca7_bcp AS c JOIN BatchCycle_V AS b ON c.name_system = b.name AND b.deleted = 0
      JOIN BCYS_BTCY_AGGC AS l ON b.id = l.dst_id AND l.src_id = @id_schedule_set AND l.deleted = 0
      WHERE  c.name_schedule_set = @name_schedule_set AND c.name_system <> '' 
      AND c.name_system IS NOT NULL
    FETCH NEXT FROM bcp INTO @name_schedule_set
  END 
  DEALLOCATE bcp 

  -- create batch job if not exists, then update the plan info for jobs using ca7_jobs table
  EXEC @rows = _CA7CreateBATC @useBCPTable = 1
  IF @rows < 0
  BEGIN
    RAISERROR('Bulk create batch job failed', 0, 1)
    ROLLBACK TRAN _DiscoverCA7Objects
    RETURN -30000
  END

  -- Finish transaction
  IF @tcount = 0
    COMMIT TRAN _DiscoverCA7Objects

  SET NOCOUNT OFF  

  IF NOT EXISTS (SELECT * FROM DiscoveryBatch 
    WHERE FormatType = @FormatType AND pstep = 3 AND error <> 0 AND error IS NOT NULL)
  BEGIN
    -- set pstep to 3, pstat = 1 for all data ready to process 
    DELETE DiscoveryBatch WHERE FormatType = @FormatType AND pstep = 3 AND pstat = 1 
    RETURN 0
  END
  ELSE
  BEGIN
    RAISERROR('Please check DiscoveryBatch table for error messages', 0, 1)
    RETURN -30000
  END

GO

-------------------------------------------------------------------------------------------------------------
