-- $Header:   //ARCHIVE/Archive/DB/sql/CA7EventHandling.sqv   1.21   Feb 07 2001 11:50:10   Xiaoyan  $
--
-- CA7EventHandling.sql
--
-- This file defines data type, format type, action type, event mapping, event handling 
-- and Staged processes for CA7 data loading
--

USE Object
GO

---------------------------------------------------------------------------------------------------
-- add the format type and action type
SET NOCOUNT ON 
EXEC asisp_defineFormatType 11, 'CA7', 'Events from CA7'
EXEC asisp_defineActionType 11, 1, 'STATE_CHANGE', 'State Change events from CA7'

-- add the data types
EXEC asisp_defineDataType 11, 0, 'DATE_TIME', 'Date and time of message'
EXEC asisp_defineDataType 11, 1, 'SEQ_NO', 'Sequence Number'
EXEC asisp_defineDataType 11, 46, 'MESSAGE', 'Message text'

-- add the fields
EXEC ADD_REQUIRED_DATA_TYPE 'CA7', 'STATE_CHANGE', 'DATE_TIME'
EXEC ADD_REQUIRED_DATA_TYPE 'CA7', 'STATE_CHANGE', 'SEQ_NO'
EXEC ADD_REQUIRED_DATA_TYPE 'CA7', 'STATE_CHANGE', 'MESSAGE'

-- define event procedure mapping
EXEC asisp_defineEventProcMapping 'CA7', 'STATE_CHANGE', 'Process_CA7_Event'

-- define StagedEventProc
EXEC asisp_defineStagedEventProc 'asisp_processStagedCA7Events', 1000, 1, 1, 
'DELETE StagedCA7Event FROM ProcessedStagedEvent WHERE timestamp = evt_timestamp'

SET NOCOUNT OFF 

---------------------------------------------------------------------------------------------------
-- create event map table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'CA7EventMap' AND type = 'U')
BEGIN
  PRINT 'Creating table CA7EventMap'
  EXEC("
  CREATE TABLE CA7EventMap
  (
    id            INT IDENTITY(1,1),
    EventName     ObjName,
    WhereClause   VARCHAR(255) NOT NULL,
    SearchExp     VARCHAR(255),
    ReplaceExp    VARCHAR(255),
    MapPriority   INT,
    Enabled       TINYINT DEFAULT 1,
    StateID       INT NULL,
    AlertStateID  INT NULL,
    PriorityID    INT NULL
  )
  ")
  EXEC("
  ALTER TABLE CA7EventMap ADD
    CONSTRAINT CA7EventMap_PK PRIMARY KEY (WhereClause)
  ")
END
GO

---------------------------------------------------------------------------------------------------
-- create table StagedCA7Event
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'StagedCA7Event' AND type = 'U')
BEGIN
  PRINT 'Creating table StagedCA7Event'
  EXEC("
  CREATE TABLE StagedCA7Event
  (
      timestamp       PRIMARY KEY,
      ctime           DATETIME DEFAULT getdate(),

      parent_cid      ClassID,
      parent_id       ObjID,

      TIME            DATETIME,
      SEQ_NO          VARCHAR(50),
      MESSAGE         VARCHAR(1000)
  )
  ")
END
GO

---------------------------------------------------------------------------------------------------
-- drop table StagedCA7InProgress
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'StagedCA7InProgress' AND type = 'U')
BEGIN
  PRINT 'Creating table StagedCA7InProgress'
  EXEC('
  CREATE TABLE StagedCA7InProgress
  (
    timestamp_col   BINARY(8) PRIMARY KEY,
    ctime           DATETIME,
    src_cid         ClassID,
    src_id          ObjID,
    TIME            DATETIME NULL,
    SEQ_NO          VARCHAR(50) NULL,
    MESSAGE         VARCHAR(500) NULL,

    bcys_id         ObjID DEFAULT 0,
    btcy_id         ObjID DEFAULT 0,
    batc_id         ObjID DEFAULT 0,

    MSG             VARCHAR(50) NULL,
    JOBNAME         VARCHAR(50) NULL,
    JOBNUM          VARCHAR(50) NULL,
    SCHEDID         INT DEFAULT 0,
    SYSTEMNAME      VARCHAR(50) NULL,
    CONDITIONCODE   VARCHAR(50) NULL,
    MESSAGEDETAIL   VARCHAR(256) NULL,
    JCLID           VARCHAR(50) NULL,
    REQUIREDBY      VARCHAR(50) NULL,
    MUSTSTARTBY     VARCHAR(50) NULL,
    DUEOUT          VARCHAR(50) NULL
  )')
END
GO

---------------------------------------------------------------------------------------------------
-- drop old table
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ca7_discarded_events' AND type = 'U')
  DROP TABLE ca7_discarded_events
GO

-- create table ca7_discarded_SIRD11 for discarded events
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'ca7_discarded_SIRD11' AND type = 'U')
BEGIN
  PRINT 'Creating table ca7_discarded_SIRD11'
  EXEC('
  CREATE TABLE ca7_discarded_SIRD11
  (
    bcys_id ObjID PRIMARY KEY (bcys_id, job_name, job_num),
    job_name ObjName,
    job_num INT
  )'
  )
END
GO

---------------------------------------------------------------------------------------------------
-- create table ca7_SIRD11_events for SIRD-11 events
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'ca7_SIRD11_events' AND type = 'U')
BEGIN
  PRINT 'Creating table ca7_SIRD11_events'
  EXEC('
  CREATE TABLE ca7_SIRD11_events
  (
    bcys_id ObjID PRIMARY KEY(bcys_id, job_num, job_name),
    batc_id ObjID,
    job_name ObjName,
    job_num INT
  )')
END 
GO
---------------------------------------------------------------------------------------------------
-- create table ca7_audit_events
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'ca7_audit_events' AND type = 'U')
BEGIN
  PRINT 'Creating table ca7_audit_events'
  EXEC('
  CREATE TABLE ca7_audit_events
  (
    src_cid ClassID,
    src_id ObjID,
    ctime DATETIME,
    MESSAGE VARCHAR(400)
  )')
END 
GO

---------------------------------------------------------------------------------------------------
-- _CreateCA7EventMapEntry
PRINT 'Creating procedure _CreateCA7EventMapEntry'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_CreateCA7EventMapEntry' AND type = 'P')
  DROP PROCEDURE _CreateCA7EventMapEntry
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7EventHandling.sqv  $
-- $Revision:   1.21  $
-- $Date:   Feb 07 2001 11:50:10  $
-- $Author:   Xiaoyan  $
-- Creates or updates a CA7EventMap entry
CREATE PROCEDURE _CreateCA7EventMapEntry
  @cid            ClassID = NULL,
  @id             ObjID = NULL,
  @EventName      VARCHAR(255),
  @WhereClause    VARCHAR(255),
  @SearchExp      VARCHAR(255),
  @ReplaceExp     VARCHAR(255),
  @MapPriority    INT,
  @Enabled        TINYINT = 1,
  @StateID        INT = NULL,
  @AlertStateID   INT = NULL,
  @PriorityID     INT = NULL
AS
  IF NOT EXISTS (SELECT * FROM CA7EventMap WHERE WhereClause = @WhereClause)
    INSERT CA7EventMap (EventName, WhereClause, SearchExp, ReplaceExp, MapPriority, Enabled, StateID, AlertStateID, PriorityID)
    SELECT @EventName, @WhereClause, @SearchExp, @ReplaceExp, @MapPriority, @Enabled, @StateID, @AlertStateID, @PriorityID
  ELSE
    UPDATE CA7EventMap SET
      EventName = @EventName,
      SearchExp = @SearchExp,
      ReplaceExp = @ReplaceExp,
      MapPriority = @MapPriority,
      Enabled = @Enabled,
      StateID = @StateID,
      AlertStateID = @AlertStateID,
      PriorityID = @PriorityID
    WHERE
      WhereClause = @WhereClause
GO

SET NOCOUNT ON 
-- SIRD-11
EXEC _CreateCA7EventMapEntry
  @EventName = 'Execution Requirements',
  @WhereClause = 'SIRD-11%',
  @SearchExp = '\([^ ][^ ]*\) \**[ ]JOB=\([^ ][^ ][^(]*\) (\([0-9][0-9]*\)) .*JCLID=\([0-9][0-9]*\) SYSTEM=\([^ ]*\) DESC=.*DUE-OUT[ ]\([0-9][0-9].[0-9]*\)/\([0-9][0-9]\)\([0-9][0-9]\) .*[ ]SCHID=\([0-9][0-9]*\).*',
  @ReplaceExp = "MSG = '\1', JOBNAME = '\2', JOBNUM = '\3', JCLID = '\4', SYSTEMNAME = '\5', DUEOUT = '\6' + ' \7' + ':' + '\8', SCHEDID = '\9'",
  @MapPriority = 2

-- SIRD-11 handle NON-EXECUTABLE jobs
EXEC _CreateCA7EventMapEntry
  @EventName = 'Execution Requirements',
  @WhereClause = 'SIRD-11%NON-EXECUTABLE%',
  @SearchExp = '\([^ ][^ ]*\) \**[ ]*JOB[= ]\([^ ][^ ][^(]*\) (\([0-9][0-9]*\)) .*SYSTEM=\([^ ]*\) DESC=.*DUE-OUT[ ]\([0-9][0-9].[0-9]*\)/\([0-9][0-9]\)\([0-9][0-9]\) .*[ ]SCHID=\([0-9][0-9]*\).*',
  @ReplaceExp = "MSG = '\1', JOBNAME = '\2', JOBNUM = '\3', SYSTEMNAME = '\4', DUEOUT = '\5' + ' \6' + ':' + '\7', SCHEDID = '\8', JCLID = 'NON-EXECUTABLE'",
  @MapPriority = 3

-- SMF0-12
EXEC _CreateCA7EventMapEntry
  @EventName = 'Status',
  @WhereClause = 'SMF0-12%',
  @SearchExp = '\([^ ][^ ]*\) JOB[= ]\([^ ][^ ][^(]*\) (\([0-9][0-9]*\)) \(.*\)\..*',
  @ReplaceExp = "MSG = '\1', JOBNAME = '\2', JOBNUM = '\3', MESSAGEDETAIL = '\4'",
  @MapPriority = 2

-- SCRJ-12
EXEC _CreateCA7EventMapEntry
  @EventName = 'Status',
  @WhereClause = 'SCRJ-12%',
  @SearchExp = '\([^ ][^ ]*\) [ ]*JOB[= ]\([^ ][^ ][^(]*\) (\([0-9][0-9]*\)) \([^\.][^\.]*\)\..* HIGHEST CONDITION CODE = \([0-9][0-9]*\)\..*',
  @ReplaceExp = "MSG = '\1', JOBNAME = '\2', JOBNUM = '\3', MESSAGEDETAIL = '\4', CONDITIONCODE = '\5'",
  @MapPriority = 3

-- SCNP-11 (without a 'MUST START BY' clause)
EXEC _CreateCA7EventMapEntry
  @EventName = 'Status',
  @WhereClause = 'SCNP-11%',
  @SearchExp = '\([^ ][^ ]*\) JOB[= ]\([^ ][^ ][^(]*\)(\([0-9][0-9]*\)) SYS=\([^ ]*\) \(.*\) SCHID=\([0-9]*\).*',
  @ReplaceExp = "MSG = '\1', JOBNAME = '\2', JOBNUM = '\3', SYSTEMNAME = '\4', MESSAGEDETAIL = '\5', SCHEDID = '\6'",
  @MapPriority = 3

-- SMF0-19
EXEC _CreateCA7EventMapEntry
  @EventName = 'Abending',
  @WhereClause = 'SMF0-19%',
  @SearchExp = "\([^ ][^ ]*\) JOB[= ]\([^ ][^ ][^(]*\)(\([0-9][0-9]*\)) \(.*\)",
  @ReplaceExp = "MSG = '\1', JOBNAME = '\2', JOBNUM = '\3', MESSAGEDETAIL = '\4'",
  @MapPriority = 2

-- Unmatched events
EXEC _CreateCA7EventMapEntry
  @EventName = 'UNMATCHED',
  @WhereClause = '%',
  @SearchExp = "\(.*\)",
  @ReplaceExp = "MESSAGE = '1'",
  @MapPriority = 1

SET NOCOUNT OFF

---------------------------------------------------------------------------------------------------
-- Process_CA7_Event
PRINT 'Creating procedure Process_CA7_Event'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'Process_CA7_Event' AND type = 'P')
  DROP PROCEDURE Process_CA7_Event
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7EventHandling.sqv  $
-- $Revision:   1.21  $
-- $Date:   Feb 07 2001 11:50:10  $
-- $Author:   Xiaoyan  $
-- Inserts a CA7 message into the StagedCA7Event table
-- CA7 auto-discovery
CREATE PROCEDURE Process_CA7_Event
  @SourceCID          ClassID,
  @SourceID           ObjID,
  @FormatType         SMALLINT,
  @ActionType         SMALLINT,
  @DATE_TIME          VARCHAR(30) = NULL,
  @SEQ_NO             VARCHAR(50) = NULL,
  @MESSAGE            VARCHAR(400) = NULL
AS
  -- through away unwanted events
  IF (@MESSAGE like '%SPO7-10%') OR (@MESSAGE like '%SATO-15%')
  BEGIN
    RAISERROR('Through away SPO7-10 and SATO-15 messages.', 0, 1)
    RETURN
  END
  DECLARE @datetime DATETIME
  EXEC asisp_convertMVSDateTime @DATE_TIME, @datetime OUTPUT
  INSERT StagedCA7Event (parent_cid, parent_id, TIME, SEQ_NO, MESSAGE)
    SELECT @SourceCID, @SourceID, @datetime, @SEQ_NO, @MESSAGE
  RETURN
GO

---------------------------------------------------------------------------------------------------
-- create asisp_setCA7Globals procedure
PRINT 'Creating procedure asisp_setCA7Globals'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_setCA7Globals')
  DROP PROCEDURE asisp_setCA7Globals
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7EventHandling.sqv  $
-- $Revision:   1.21  $
-- $Date:   Feb 07 2001 11:50:10  $
-- $Author:   Xiaoyan  $
-- set up global variable to handle NULL SYSTEM field in SIRD-11 evnets. For propertyname = 'keep_unknown'
-- and groupname = 'CA7', if propertyvalue = 1, create all jobs under '*UNKNOWN*' schedule as non-key, 
-- If propertyvalue = 0, discard the events.  
CREATE PROCEDURE asisp_setCA7Globals
@groupname VARCHAR(50), 
@propertyname VARCHAR(50), 
@propertyvalue VARCHAR(50), 
@propertydatatype VARCHAR(50)
AS

  RAISERROR('Set Global variable %s for %s', 0, 1, @propertyname, @groupname) 
  SET NOCOUNT ON 

  IF NOT EXISTS (SELECT * FROM SystemConfiguration 
                   WHERE groupname = @groupname and propertyname = @propertyname)
    INSERT SystemConfiguration(groupname, propertyname, propertyvalue, propertydatatype)
      SELECT @groupname, @propertyname, @propertyvalue, @propertydatatype
 
  ELSE 
    UPDATE SystemConfiguration SET groupname = @groupname, 
      propertyname = @propertyname, 
      propertyvalue = @propertyvalue, 
      propertydatatype = @propertydatatype

  SET NOCOUNT OFF
GO

-- Create default global variable for ca7 keep_unknown
EXEC asisp_setCA7Globals 'CA7', 'keep_unknown', '0', 'TINYINT' 
GO

---------------------------------------------------------------------------------------------------
-- asisp_ca7auditevents
PRINT 'Creating procedure asisp_ca7auditevents'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_ca7auditevents' AND type = 'P')
  DROP PROCEDURE asisp_ca7auditevents
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7EventHandling.sqv  $
-- $Revision:   1.21  $
-- $Date:   Feb 07 2001 11:50:10  $
-- $Author:   Xiaoyan  $
-- handle ca7 events which cannot be mapped with current defination
CREATE PROCEDURE asisp_ca7auditevents
  @keep_unknown INT = 0
AS

  RAISERROR('Execute asisp_ca7auditevents.', 0, 1)

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_ca7auditevents
  ELSE
    SAVE TRANSACTION asisp_ca7auditevents

  -- insert unmatched event into audit table
  IF EXISTS ( SELECT * FROM StagedCA7InProgress WHERE MSG = '' OR MSG IS NULL)
  BEGIN
    INSERT ca7_audit_events 
      SELECT src_cid, src_id, ctime, MESSAGE 
      FROM StagedCA7InProgress
      WHERE MSG = '' OR MSG IS NULL
    -- insert ProcessedStagedEvents
    INSERT ProcessedStagedEvent 
      SELECT timestamp_col, getdate() FROM StagedCA7InProgress
      WHERE MSG = '' OR MSG IS NULL
    -- delete from StagedCA7InProgress
    DELETE StagedCA7InProgress WHERE MSG = '' OR MSG IS NULL
  END
  IF @tcount = 0
    COMMIT TRAN asisp_ca7auditevents

GO

---------------------------------------------------------------------------------------------------
-- asisp_lookupCA7BCYSID
PRINT 'Creating procedure asisp_lookupCA7BCYSID'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_lookupCA7BCYSID' AND type = 'P')
  DROP PROCEDURE asisp_lookupCA7BCYSID
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7EventHandling.sqv  $
-- $Revision:   1.21  $
-- $Date:   Feb 07 2001 11:50:10  $
-- $Author:   Xiaoyan  $
-- Look up ca7 schedule set ids based on os_id from LOB defination for "CA7 COMPLEXES" 
CREATE PROCEDURE asisp_lookupCA7BCYSID
AS

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_lookupCA7BCYSID
  ELSE
    SAVE TRANSACTION asisp_lookupCA7BCYSID

  -- create temp table to lookup schedule set ids based on os_ids
  CREATE TABLE #ca7_bcys
  (
    os_id 	ObjID PRIMARY KEY, 
    os_name 	ObjName NULL, 
    bcys_name 	ObjName NULL
  )

  INSERT #ca7_bcys(os_id) SELECT DISTINCT src_id FROM StagedCA7InProgress
  UPDATE t SET t.os_name = o.name FROM #ca7_bcys AS t 
    JOIN OS_V AS o ON t.os_id = o.id
    JOIN containment AS c ON o.id = c.dst_id AND c.dst_cid = 'OS' AND c.deleted = 0

  UPDATE t SET t.bcys_name = lob2.name FROM #ca7_bcys AS t
    JOIN link AS l ON t.os_id = l.dst_id AND l.dst_cid = 'OS' AND l.link_type = 'LOBL' AND l.src_cid ='LOB' AND l.deleted = 0
    JOIN LineOfBusiness_V AS lob ON t.os_name = lob.name AND l.src_id = lob.id
    JOIN LOB_LOB_LOBC AS l2 ON l2.dst_id = l.src_id AND l2.deleted = 0
    JOIN LOB_LOB_LOBC AS l3 ON l3.dst_id = l2.src_id AND l3.deleted = 0
    JOIN LineOfBusiness_V AS lob2 ON l2.src_id = lob2.id AND lob2.deleted = 0     
    JOIN LineOfBusiness_V AS lob3 ON l3.src_id = lob3.id AND lob3.name = 'CA7 COMPLEXES' AND lob3.deleted = 0 

  -- update schedule set ids for defined from LOB
  UPDATE s SET s.bcys_id = bcys.id FROM StagedCA7InProgress AS s
    JOIN #ca7_bcys AS t ON s.src_id = t.os_id
    JOIN BatchCycles_V AS bcys ON bcys.name = t.bcys_name 
    JOIN link AS l ON bcys.id = l.dst_id AND l.dst_cid = 'BCYS' AND l.link_type = 'PHYC' AND l.deleted = 0 
    WHERE t.bcys_name IS NOT NULL    

  -- handle unspecified ca7 schedule sets by os
  DECLARE @os_id ObjID
  DECLARE os CURSOR FOR SELECT DISTINCT src_id FROM StagedCA7InProgress
    WHERE bcys_id IS NULL OR bcys_id = 0
  OPEN os
  FETCH NEXT FROM os INTO @os_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    -- create CA7 unknown OS Exception on OS
    INSERT INTO StagedEXCP (
		parent_cid, 
		parent_id, 
		TIME, 
		EXCP_NAME, 
		EXCP_CD, 
		AlertState, 
		Priority,
		nEXCH)
       SELECT 'OS', 
 		@os_id, 
		getdate(),
		'CA7_UNKNOWN_OS', 
		'The link between schedule set and OS was not set in LOB',
		2,
		3,
		1 
    -- If we don't want to through ou the events and let these events set in 
    -- StagedCA7Event table untill the required config is set, following command need to be blocked.      
    INSERT ProcessedStagedEvent 
      SELECT timestamp_col, getdate() 
      FROM StagedCA7InProgress 
      WHERE src_id = @os_id
   
    -- through away the events for unspecified ca7 schedule sets
    DELETE StagedCA7InProgress WHERE src_id = @os_id
    FETCH NEXT FROM os INTO @os_id
  END
  DEALLOCATE os

  -- make sure all bcys_id is not null
  IF EXISTS (SELECT * FROM StagedCA7InProgress WHERE bcys_id = 0)
  BEGIN
    RAISERROR('Ca7 batch schedule set id is not found.', 0, 1)
    RETURN -30000 
  END
  -- Finish up
  IF @tcount = 0
    COMMIT TRAN asisp_lookupCA7BCYSID

GO

---------------------------------------------------------------------------------------------------
-- asisp_ca7discardedevents
PRINT 'Creating procedure asisp_ca7discardedevents'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_ca7discardedevents' AND type = 'P')
  DROP PROCEDURE asisp_ca7discardedevents
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7EventHandling.sqv  $
-- $Revision:   1.21  $
-- $Date:   Feb 07 2001 11:50:10  $
-- $Author:   Xiaoyan  $
-- Process ca7 discarded events 
CREATE PROCEDURE asisp_ca7discardedevents
AS

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_ca7discardedevents
  ELSE
    SAVE TRANSACTION asisp_ca7discardedevents

  -- unset ca7_discarded_SIRD11 table for all new SIRD-11 based on JOBNUM
  DELETE c FROM ca7_discarded_SIRD11 AS c 
    JOIN StagedCA7InProgress AS s ON c.bcys_id = s.bcys_id AND c.job_num = s.JOBNUM
    AND s.MSG = 'SIRD-11'
  -- insert new entries for SIRD-11 without SYSTEMNAME to ca7_discarded_SIRD11 table 
  INSERT ca7_discarded_SIRD11 
    SELECT bcys_id, JOBNAME, CONVERT(INT, JOBNUM) 
    FROM StagedCA7InProgress 
    WHERE MSG = 'SIRD-11' AND (SYSTEMNAME IS NULL OR SYSTEMNAME = '')
  -- insert NON-EXECUTABLE event into ca7_discarded_SIRD11
  INSERT ca7_discarded_SIRD11 
    SELECT bcys_id, JOBNAME, CONVERT(INT, JOBNUM) 
    FROM StagedCA7InProgress
    WHERE JCLID = 'NON-EXECUTABLE'
  -- delete events for bcys_id and job_num in ca7_discarded_SIRD11 
  INSERT ProcessedStagedEvent 
    SELECT s.timestamp_col, getdate() 
    FROM StagedCA7InProgress AS s
    JOIN ca7_discarded_SIRD11 AS c ON s.bcys_id = c.bcys_id 
    AND s.JOBNUM = c.job_num
    AND s.JOBNAME = c.job_name

  DELETE s FROM StagedCA7InProgress AS s
    JOIN ca7_discarded_SIRD11 AS c ON s.bcys_id = c.bcys_id 
    AND s.JOBNUM = c.job_num
    AND s.JOBNAME = c.job_name

  -- finish up
  IF @tcount = 0
    COMMIT TRAN asisp_ca7discardedevents
  RETURN 0

GO

---------------------------------------------------------------------------------------------------
-- asisp_processca7SIRD11Events
PRINT 'Creating procedure asisp_processca7SIRD11Events'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processca7SIRD11Events' AND type = 'P')
  DROP PROCEDURE asisp_processca7SIRD11Events
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7EventHandling.sqv  $
-- $Revision:   1.21  $
-- $Date:   Feb 07 2001 11:50:10  $
-- $Author:   Xiaoyan  $
-- Process ca7 SIRD-11 events and events with schedule name. 
-- It is required SIRD-11 event has schedule name 
CREATE PROCEDURE asisp_processca7SIRD11Events
AS

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_processca7SIRD11Events
  ELSE
    SAVE TRANSACTION asisp_processca7SIRD11Events

  -- Update btcy_id for existing schedules
  UPDATE s SET s.btcy_id = b.id FROM StagedCA7InProgress AS s
    JOIN BatchCycle_V AS b ON s.SYSTEMNAME = b.name AND MSG = 'SIRD-11'
    JOIN link AS l ON l.dst_id = b.id AND l.src_id = s.bcys_id 
    WHERE l.src_cid = 'BCYS' AND l.dst_cid = 'BTCY' AND link_type = 'AGGC' AND l.deleted = 0 

  -- create non existing schedules
  DECLARE @rows int
  DELETE ca7_schedules WHERE bulk_load = 0
  -- get a list of the batch schedule into ca7_schedules table with _ParentID
  INSERT INTO ca7_schedules (name, bcys_id, name_schedule_set, bulk_load)
    SELECT DISTINCT SYSTEMNAME, bcys_id, '', 0
    FROM StagedCA7InProgress 
    WHERE MSG = 'SIRD-11' 
    AND SYSTEMNAME IS NOT NULL 
    AND SYSTEMNAME <> '' 
    AND btcy_id = 0
  IF EXISTS (SELECT * FROM ca7_schedules WHERE bulk_load = 0) 
  BEGIN
    EXEC @rows = _LoadCA7Plan_CreateBTCY NULL, NULL, @useBCPTable = 0
    IF @rows < 0
    BEGIN
      RAISERROR('Bulk create batch schedules failed', 0, 1)
      ROLLBACK TRAN _LoadCA7Plan_ByBcp
      RETURN -30000
    END
    -- update btcy_id for new created objects in StagedCA7InProgress
    UPDATE s SET s.btcy_id = b.id FROM StagedCA7InProgress AS s
      JOIN BatchCycle_V AS b ON s.SYSTEMNAME = b.name
      JOIN link AS l ON l.dst_id = b.id AND l.src_id = s.bcys_id 
      AND l.src_cid = 'BCYS' AND l.dst_cid = 'BTCY' AND l.link_type = 'AGGC' AND l.deleted = 0
      WHERE s.btcy_id = 0
  END

  -- Update batc_id of SIRD-11 events
  UPDATE s SET s.batc_id = b.id FROM  StagedCA7InProgress AS s 
    JOIN BATCH_V AS b ON s.JOBNAME = b.name AND s.MSG = 'SIRD-11'
    AND s.SCHEDID = b._SCHED_PACK
    JOIN link AS l ON b.id = l.dst_id 
    AND l.src_id = s.btcy_id 
    WHERE l.src_cid = 'BTCY' 
    AND l.dst_cid = 'BATC' 
    AND l.deleted = 0

  -- discover non-exist BATC objects, for jobs run multiple times a day, 
  -- use event start time and earlest DUEOUT   
  DELETE ca7_jobs WHERE bulk_load = 0
  CREATE TABLE #temp_job(
    JOBNAME 	ObjName,
    SYSTEMNAME 	ObjName,
    btcy_id 	ObjID,
    SCHEDID	ObjID,
    start_year 	INT,
    start_day	INT,
    start_time	CHAR(5),
    end_year	INT,
    end_day	INT,
    end_time	CHAR(5)
  )

  INSERT #temp_job 
    SELECT JOBNAME, 
      	SYSTEMNAME, 
      	btcy_id, 
	SCHEDID,
        DATEPART(yy, TIME) AS start_year,
        DATEPART(dy, TIME) AS start_day,
        CONVERT(CHAR(5), TIME, 108) AS start_time,                
        2000 + CONVERT(INT, SUBSTRING(DUEOUT, 1, 2)) AS end_year,
        CONVERT(INT, SUBSTRING(DUEOUT, 4, 3)) AS end_day,
        SUBSTRING(DUEOUT, CHARINDEX(' ', DUEOUT)+1, 5) AS end_time          
    FROM StagedCA7InProgress 
      WHERE MSG = 'SIRD-11' AND batc_id = 0 AND SYSTEMNAME IS NOT NULL AND SYSTEMNAME <> ''

  DELETE t1 FROM #temp_job AS t1 
    JOIN #temp_job AS t2 ON t1.JOBNAME = t2.JOBNAME
    AND t1.SYSTEMNAME = t2.SYSTEMNAME
    AND t1.btcy_id = t2.btcy_id
    AND t1.SCHEDID = t2.SCHEDID
    AND (t1.end_day < t2.end_day OR CONVERT(datetime, t1.end_time) < CONVERT(datetime, t2.end_time))
    
  INSERT INTO ca7_jobs (name, cycle_name, btcy_id, sched_id, flag_keyjob, year_start, day_start, time_start, year_stop, day_stop, time_stop, bulk_load)
    SELECT DISTINCT JOBNAME, 
      	SYSTEMNAME, 
      	btcy_id, 
	SCHEDID,
	0,
        start_year,
        start_day,
        start_time,
        end_year,
        end_day,
        end_time, 
        0
      FROM #temp_job

  IF EXISTS (SELECT * FROM ca7_jobs WHERE bulk_load = 0)
  BEGIN
    EXEC @rows = _CA7CreateBATC @useBCPTable = 0
    IF @rows < 0
    BEGIN
      RAISERROR('Bulk create batch jobs failed when loading events', 0, 1) 
      ROLLBACK TRAN asisp_processca7SIRD11Events
      RETURN -30000
    END
    -- Update batc_id for new created objects
    UPDATE s SET s.batc_id = b.id FROM  StagedCA7InProgress AS s 
      JOIN BATCH_V AS b ON s.JOBNAME = b.name 
      AND s.SCHEDID = b._SCHED_PACK
      JOIN link AS l ON b.id = l.dst_id 
      AND l.src_id = s.btcy_id 
      AND l.src_cid = 'BTCY' 
      AND l.dst_cid = 'BATC'
      AND l.deleted = 0
      WHERE s.batc_id = 0 AND s.MSG = 'SIRD-11'
  END

  -- Unset ca7_SIRD11_events
  DELETE c FROM ca7_SIRD11_events AS c
    JOIN StagedCA7InProgress AS s ON c.bcys_id = s.bcys_id AND c.job_num = s.JOBNUM AND c.job_name = s.JOBNAME
    WHERE s.MSG = 'SIRD-11'

  -- set ca7_SIRD11_events for new SIRD-11 events
  INSERT ca7_SIRD11_events 
    SELECT bcys_id, batc_id, JOBNAME, JOBNUM
    FROM StagedCA7InProgress 
    WHERE MSG = 'SIRD-11'
  
  -- update mtime in BATCH_C for delete old objects
  UPDATE BATCH_C SET mtime = getdate() 
    WHERE id IN (SELECT DISTINCT batc_id FROM StagedCA7InProgress WHERE MSG = 'SIRD-11' AND batc_id <> 0)

  IF @tcount = 0
    COMMIT TRAN asisp_processca7SIRD11Events
GO

---------------------------------------------------------------------------------------------------
-- asisp_processca7nonSIRD11Events
PRINT 'Creating procedure asisp_processca7nonSIRD11Events'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processca7nonSIRD11Events' AND type = 'P')
  DROP PROCEDURE asisp_processca7nonSIRD11Events
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7EventHandling.sqv  $
-- $Revision:   1.21  $
-- $Date:   Feb 07 2001 11:50:10  $
-- $Author:   Xiaoyan  $
-- Handle ca7 events without system name. JOBNUM and JOBNAME are used to lookup from ca7_SIRD11_events
--  on the schedule set
CREATE PROCEDURE asisp_processca7nonSIRD11Events
AS

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_processca7nonSIRD11Events
  ELSE
    SAVE TRANSACTION asisp_processca7nonSIRD11Events

  -- lookup batc_id and btcy_id for JOBNUM matches in database under the same schedule set
  UPDATE s SET s.batc_id = c.batc_id FROM StagedCA7InProgress AS s
    JOIN ca7_SIRD11_events AS c ON s.bcys_id = c.bcys_id 
    AND s.JOBNAME = c.job_name
    AND s.JOBNUM = c.job_num

  -- If JOBNUM and JOBNAME are not in ca7_SIRD11_events, through exception 'CA7_Missing_SIRD11' 
  -- on schedule set  
  -- create CA7 Missing SIRD11 Exception on the schdule set for the job
  INSERT INTO StagedEXCP (
		parent_cid, 
		parent_id, 
		TIME, 
		EXCP_NAME, 
		EXCP_CD, 
		AlertState, 
		Priority,
		nEXCH)
     SELECT 'BCYS', 
 		bcys_id, 
		getdate(),
		'CA7_Missing_SIRD11', 
                'SIRD-11 event was not sent for schedule = ' + SYSTEMNAME + ', job= ' + JOBNAME + '.',
		2,
		3,
		1 
     FROM StagedCA7InProgress 
     WHERE batc_id = 0 OR batc_id IS NULL

  -- update start time and unset end time for SMF0-12
--  UPDATE b SET b._StartTime = getdate(), b._StopTime = '' FROM BATCH_V AS b
--    WHERE id in (SELECT batc_id FROM StagedCA7InProgress WHERE MSG = 'SMF0-12')

  -- update stop time based on SCRJ-12 and SMF0-19
--  UPDATE b SET b._StopTime = getdate() FROM BATCH_V AS b
--    WHERE id in (SELECT batc_id FROM StagedCA7InProgress WHERE MSG = 'SMF0-19' OR MSG = 'SCRJ-12')

  UPDATE b SET b._StartTime = s.TIME, b._StopTime = '' FROM BATCH_V AS b
    JOIN StagedCA7InProgress AS s ON b.id = s.batc_id AND s.MSG = 'SMF0-12'

  -- update stop time based on SCRJ-12 and SMF0-19
  UPDATE b SET b._StopTime = s.TIME FROM BATCH_V AS b
    JOIN StagedCA7InProgress AS s ON b.id = s.batc_id 
    AND (s.MSG = 'SMF0-19' OR s.MSG = 'SCRJ-12')

  -- clean up for NON SIRD-11 events
  INSERT ProcessedStagedEvent 
    SELECT timestamp_col, getdate() FROM StagedCA7InProgress 
    WHERE batc_id = 0 OR batc_id IS NULL

  DELETE StagedCA7InProgress WHERE batc_id = 0 OR batc_id IS NULL
 
  IF @tcount = 0
    COMMIT TRAN asisp_processca7nonSIRD11Events
GO

---------------------------------------------------------------------------------------------------
-- asisp_processStagedCA7Events
PRINT 'Creating procedure asisp_processStagedCA7Events'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processStagedCA7Events' AND type = 'P')
  DROP PROCEDURE asisp_processStagedCA7Events
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7EventHandling.sqv  $
-- $Revision:   1.21  $
-- $Date:   Feb 07 2001 11:50:10  $
-- $Author:   Xiaoyan  $
-- Processes events in the StagedCA7Event table
CREATE PROCEDURE asisp_processStagedCA7Events
  @maxrows INT = 0,
  @verbose TINYINT = 0
AS

  SET NOCOUNT ON
  TRUNCATE TABLE StagedCA7InProgress
  DECLARE @err INT, @rc INT, @step INT
  SET @step = 1

  SET ROWCOUNT @maxrows

  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': creating StagedCA7InProgress'
  SET @step = @step + 1

  DECLARE @nEvents INT
  INSERT INTO StagedCA7InProgress (timestamp_col, ctime, src_cid, src_id, TIME, SEQ_NO, MESSAGE)
  SELECT * FROM StagedCA7Event
  SELECT @nEvents = @@ROWCOUNT

  IF @verbose = 1
  BEGIN
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': Inserted ' + CAST(@nEvents AS VARCHAR(9)) + ' row(s)' 
    SELECT @step AS Step, * FROM StagedCA7InProgress ORDER BY timestamp_col
  END
  SET @step = @step + 1

  SET ROWCOUNT 0

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_processStagedCA7Events
  ELSE
    SAVE TRANSACTION asisp_processStagedCA7Events

  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': processing message fields.'
  SET @step = @step + 1

  -- We now iterate through each message and find the best matching CA7EventMap entry based
  -- on the CA7EventMap.WhereClause column. In the case of multiple WhereClause matches we 
  -- use the CA7EventMap.MapPriority column to determine which one applies. If more than
  -- one row with a matching WhereClause and a given MapPriority are found then which is
  -- chosen will be indeterminate.

  -- Create a regular expression context handle. This will be reused for each SearchExp
  DECLARE @regex int, @hr int
  EXEC @hr = sp_OACreate 'TMRegExp.RegExContext', @regex OUT
  IF @hr <> 0
  BEGIN
    DECLARE @errtext VARCHAR(255)
    EXEC sp_OAGetErrorInfo NULL, NULL, @errtext OUT
    RAISERROR('%s', 3, 1, @errtext)
    ROLLBACK TRANSACTION asisp_processStagedCA7Events
    RETURN -30000
  END

  DECLARE @MESSAGE VARCHAR(500), @SearchExp VARCHAR(255), @ReplaceExp VARCHAR(255)
  DECLARE cursor_asisp_processStagedCA7Events CURSOR FOR 
    SELECT MESSAGE FROM StagedCA7InProgress
  OPEN cursor_asisp_processStagedCA7Events
  FETCH NEXT FROM cursor_asisp_processStagedCA7Events INTO @MESSAGE
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    -- Find the best matching CA7EventMap row
    DECLARE @priority INT
    SELECT @priority = MAX(MapPriority) FROM CA7EventMap WHERE @MESSAGE LIKE WhereClause AND Enabled = 1
    SELECT @SearchExp = SearchExp, @ReplaceExp = ReplaceExp FROM CA7EventMap WHERE @MESSAGE LIKE WhereClause AND Enabled = 1 AND MapPriority = @priority
    IF @@ROWCOUNT <> 1
    BEGIN
      RAISERROR('No mapping for message text: %s, aborting.', 3, 1, @MESSAGE)
      DEALLOCATE cursor_asisp_processStagedCA7Events
      ROLLBACK TRANSACTION asisp_processStagedCA7Events
      RETURN -30000
    END

    -- Compile the search regular expression
    EXEC @hr = sp_OAMethod @regex, 'Compile', NULL, @SearchExp, 0
    IF @hr <> 0
    BEGIN
      EXEC sp_OAGetErrorInfo @regex, NULL, @errtext OUT
      RAISERROR('%s', 3, 1, @errtext)
      DEALLOCATE cursor_asisp_processStagedCA7Events
      ROLLBACK TRANSACTION asisp_processStagedCA7Events
      RETURN -30000
    END

    -- Process the regular expression on the message text
    EXEC @hr = sp_OAMethod @regex, 'Exec', NULL, @MESSAGE, 0
    IF @hr <> 0
    BEGIN
      EXEC sp_OAGetErrorInfo @regex, NULL, @errtext OUT
      RAISERROR('%s', 3, 1, @errtext)
      DEALLOCATE cursor_asisp_processStagedCA7Events
      ROLLBACK TRANSACTION asisp_processStagedCA7Events
      RETURN -30000
    END

    -- Replace the matched fields from the regular expression using the replace expression
    -- in order to create the update statement which will set the various fields of the
    -- current row
    DECLARE @processing_sql VARCHAR(2000), @update_sql VARCHAR(1000)
    EXEC @hr = sp_OAMethod @regex, 'Replace', @update_sql OUT, @ReplaceExp
    IF @hr <> 0
    BEGIN
      EXEC sp_OAGetErrorInfo @regex, NULL, @errtext OUT
      RAISERROR('%s', 3, 1, @errtext)
      DEALLOCATE cursor_asisp_processStagedCA7Events
      ROLLBACK TRANSACTION asisp_processStagedCA7Events
      RETURN -30000
    END

    -- Use dynamic SQL to update the current row of the cursor
    SELECT @processing_sql = "UPDATE StagedCA7InProgress SET " + @update_sql + " WHERE CURRENT OF cursor_asisp_processStagedCA7Events"
    EXEC(@processing_sql)
    UPDATE StagedCA7InProgress SET SYSTEMNAME = NULL WHERE SYSTEMNAME = ' '

    FETCH NEXT FROM cursor_asisp_processStagedCA7Events INTO @MESSAGE
  END
  DEALLOCATE cursor_asisp_processStagedCA7Events

  -- handle evnet does not match any format we defined, insert events into ca7_audit_events table
  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': Insert unmapped events into audit table.'
  SET @step = @step + 1
  EXEC asisp_ca7auditevents

  -- find out schedule set id based on OS name, It is required that in LOB "CA7 COMPLEXES"
  -- was created and schedule sets of CA7 were dragged from physical tree under "CA7 COMPLEXES",
  -- Under each schedule set will be the OSs that the CA7 schedule set could run.  
  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': lookup schedule set id based on os_id'
  SET @step = @step + 1
  EXEC asisp_lookupCA7BCYSID

  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': handle CA7 discarded events.'
  SET @step = @step + 1
  -- Two look up tables(ca7_discarded_SIRD11 and ca7_SIRD11_events) for SIRD-11 events discarded events
  -- and non-discarded evnets. If @keep_unknown =1, all jobs without systemane will be created under unkmown
  -- events will be loaded into db, or else objects will not be created, events will be discarded. 
  DECLARE @keep_unknown TINYINT
  SELECT @keep_unknown = CONVERT(TINYINT, propertyvalue) 
    FROM SystemConfiguration WHERE groupname = 'CA7' AND propertyname = 'keep_unknown'
  IF @keep_unknown = 1
  -- update NULL SYSTEMNAME with '*UNKNOWN*' for SIRD-11 events. If batch job does not exist, create it
    UPDATE StagedCA7InProgress SET SYSTEMNAME = '*UNKNOWN*' 
      WHERE MSG = 'SIRD-11' AND SYSTEMNAME IS NULL
  EXEC asisp_ca7discardedevents

  -- process ca7 SIRD-11 events
  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': handle ca7 SIRD-11 events'
  SET @step = @step + 1
  DECLARE @r INT
  IF EXISTS (SELECT * FROM StagedCA7InProgress WHERE MSG = 'SIRD-11')
  BEGIN
    EXEC @r = asisp_processca7SIRD11Events
    IF @r < 0
    BEGIN
      ROLLBACK TRAN asisp_processStagedCA7Events
      RETURN @r
    END
  END 

  -- process non SIRD-11 events without SIRD-11 event in this bacth
  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': Process non SIRD-11 evnets.'
  SET @step = @step + 1
  IF EXISTS (SELECT * FROM StagedCA7InProgress WHERE MSG <> 'SIRD-11')
  BEGIN
    EXEC @r = asisp_processca7nonSIRD11Events
    IF @r < 0
    BEGIN
      ROLLBACK TRAN asisp_processStagedCA7Events
      RETURN @r
    END
  END

  -- if any objects here does not have id, there is an error
  IF EXISTS (SELECT * FROM StagedCA7InProgress WHERE batc_id = 0)
  BEGIN
    ROLLBACK TRAN asisp_processStagedCA7Events
    RAISERROR('ERROR: Not all object id is mapped.', 5, 1)
    RETURN -30000
  END

  -- create messages
  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': creating MESGs'
  SET @step = @step + 1
  UPDATE StagedCA7InProgress SET MESSAGEDETAIL = 'CA7 job_id(' + JOBNUM + '), SCHID=' + CONVERT(CHAR(3), SCHEDID) + ', '  + MESSAGEDETAIL
    WHERE MSG = 'SIRD-11' 
  UPDATE StagedCA7InProgress SET MESSAGEDETAIL = 'CA7 job_id(' + JOBNUM + '), ' + MESSAGEDETAIL
    WHERE MSG <> 'SIRD-11' 

  INSERT INTO StagedMESG (
	parent_cid, 
	parent_id, 
	TIME, 
	SEQ_NO, 
	OBJECT_NAME,
        NATIVE_KEY,
	MESSAGE, 
	REASON_CD, 
	DISCOVERY_IND,
	STATE
	)
    SELECT 'BATC', 
	s.batc_id, 
	s.TIME, 
	s.SEQ_NO, 
	s.JOBNAME,
        '',
 	s.MSG, 
	s.MESSAGEDETAIL, 
	'MON',
	t.State
    FROM StagedCA7InProgress AS s
    JOIN MessageDescription_T AS t ON s.MSG = t.MsgID AND t.obj_cid = 'BATC'
    WHERE s.batc_id <> 0 
    order by timestamp_col
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedCA7Events
    RETURN -30000
  END

  -- create exceptions
  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': creating EXCPs'
  SET @step = @step + 1
  INSERT INTO StagedEXCP (parent_cid, 
	parent_id, 
	TIME, 
	SEQ_NO, 
	OBJECT_NAME, 
	EXCP_NAME, 
	EXCP_CD, 
	AlertState, 
	Priority, 
	nEXCH)
    SELECT 'BATC', 
	s.batc_id, 
	s.TIME, 
	s.SEQ_NO, 
	s.JOBNAME, 
	s.MSG, 
	'CA7 job_id(' + s.JOBNUM + ') ' + s.MESSAGEDETAIL, 
	e.AlertStateID, 
	e.PriorityID, 
	1
    FROM StagedCA7InProgress s 
    JOIN ExceptionType_T e ON e.Exception = s.MSG AND e.cid = 'BATC'
    WHERE s.batc_id IS NOT NULL AND s.batc_id <> 0
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedCA7Events
    RETURN -30000
  END

  IF @tcount = 0
    COMMIT TRAN asisp_processStagedCA7Events

  DELETE s FROM StagedCA7Event AS s JOIN StagedCA7InProgress AS p ON s.timestamp = p.timestamp_col

  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9)) + ': processed ' + CAST(@nEvents AS VARCHAR(9)) + ' events'
  SET @step = @step + 1
  SET NOCOUNT OFF
  RETURN @nEvents

GO

---------------------------------------------------------------------------------------------------
-- asisp_cleanupCA7
PRINT 'Creating procedure asisp_cleanupCA7'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_cleanupCA7' AND type = 'P')
  DROP PROCEDURE asisp_cleanupCA7
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CA7EventHandling.sqv  $
-- $Revision:   1.21  $
-- $Date:   Feb 07 2001 11:50:10  $
-- $Author:   Xiaoyan  $
-- Clean up old ca7 objects and ca7_audit_events
CREATE PROCEDURE asisp_cleanupCA7
@days_to_keep_objs INT = 2,
@days_to_keep_audit INT = 30
AS
  RAISERROR('Cleaning up CA7 old objects and audit events', 0, 1)
  DECLARE @objs_datetime DATETIME, @events_datetime DATETIME
  SELECT @objs_datetime = DATEADD(day, -@days_to_keep_objs, getdate()),
    @events_datetime = DATEADD(day, -@days_to_keep_audit, getdate())

  -- Delete batch job objects If _AlertStateID = 1 AND _HasOwnershipNote = 0 AND events older then @objs_datetime 
  DELETE BATCH_V WHERE mtime < @objs_datetime AND _AlertStateID = 1 AND _HasOwnershipNote = 0 AND _ParentCID = 'BTCY'
  DELETE link WHERE src_cid = 'BTCY' AND dst_cid = 'BATC' AND dst_id not in 
    (SELECT id FROM BATCH_V WHERE _ParentCID = 'BTCY')
  DELETE containment WHERE src_cid = 'BTCY' AND dst_cid = 'BATC' AND dst_id not in 
    (SELECT id FROM BATCH_V WHERE _ParentCID = 'BTCY')
  --delete events for deleted objects
  DELETE Message_A WHERE _ParentCID = 'BATC' AND _ParentID not in 
    (SELECT id FROM BATCH_V WHERE _ParentCID = 'BTCY')
  DELETE Message_C WHERE id NOT IN (SELECT id FROM Message_A WHERE _ParentCID = 'BATC')
  DELETE Exception_A WHERE _ParentCID = 'BATC' AND _ParentID not in 
    (SELECT id FROM BATCH_V WHERE _ParentCID = 'BTCY')
  DELETE Exception_C WHERE id NOT IN 
    (SELECT id FROM Exception_A WHERE _ParentCID = 'BATC')

  -- delete audit events older then @events_datetime
  DELETE ca7_audit_events WHERE ctime < @events_datetime

GO
