-- This file contain CA7 default setups for:
-- Two scheduled tasks "CA7 Discovery Load" and "CA7 Discovery Process"
-- Create default global variable for ca7 set keep_unknown = 0
-- Example syntax for setup CA7 bcys name lookup table. This is customer specific
-- replace 'short_name' with batch shcedule set name on MVS and 'full_name' with
-- batch schedule set name in TBSM

-------------------------------------------------------------------------------------------------
-- Create schedule tasks for load and process ca7 autodiscovery data
USE msdb
GO

PRINT 'Creating scheduled task "CA7 Discovery Load"'
GO

IF NOT EXISTS (SELECT * FROM sysjobs WHERE name = 'CA7 Discovery Load')
BEGIN 

  DECLARE @JobID BINARY(16)
  EXEC sp_add_job @job_id = @JobID OUTPUT , 
		@job_name = N'CA7 Discovery Load', 
		@enabled = 1, 
		@owner_login_name = N'sa', 
		@description = N'', 
		@category_name = N'[Uncategorized (Local)]', 
		@notify_level_eventlog = 2, 
		@delete_level= 0
  EXEC sp_add_jobstep @job_id = @JobID, 
		@step_id = 1, 
		@step_name = N'CA7 autodiscovery step 2', 
		@command = N'EXEC _LoadCA7DiscoveryBatch', 
		@database_name = N'Object', 
		@server = N'', 
		@database_user_name = N'', 
		@subsystem = N'TSQL', 
		@cmdexec_success_code = 0, 
		@flags = 0, 
		@retry_attempts = 0, 
		@retry_interval = 1, 
		@output_file_name = N'', 
		@on_success_step_id = 0, 
		@on_success_action = 1, 
		@on_fail_step_id = 0, 
		@on_fail_action = 2
  EXEC sp_update_job @job_id = @JobID, 
		@start_step_id = 1 
  EXEC sp_add_jobschedule @job_id = @JobID, 
		@name = N'CA7 autodiscovery load', 
		@enabled = 1, 
		@freq_type = 4, 
		@active_start_date = 20010118, 
		@active_start_time = 230000, 
		@freq_interval = 1, 
		@freq_subday_type = 1, 
		@freq_subday_interval = 0, 
		@freq_relative_interval = 0, 
		@freq_recurrence_factor = 1, 
		@active_end_date = 99991231, 
		@active_end_time = 235959
  EXEC sp_add_jobserver @job_id = @JobID, 
	@server_name = N'(local)', @automatic_post = 0 
END
GO

PRINT 'Creating scheduled task "CA7 Discovery Process"'
GO

IF NOT EXISTS (SELECT * FROM sysjobs WHERE name = 'CA7 Discovery Process')
BEGIN 

  DECLARE @JobID BINARY(16)  
  EXEC sp_add_job @job_id = @JobID OUTPUT , @job_name = N'CA7 Discovery Process', @enabled = 1, @owner_login_name = N'sa', @description = N'', @category_name = N'[Uncategorized (Local)]', @notify_level_eventlog = 2, @delete_level= 0
  EXEC sp_add_jobstep @job_id = @JobID, @step_id = 1, @step_name = N'CA7 autodiscovery step 3', @command = N'EXEC _DiscoverCA7Objects', @database_name = N'Object', @server = N'', @database_user_name = N'', @subsystem = N'TSQL', @cmdexec_success_code = 0, @flags = 0, @retry_attempts = 0, @retry_interval = 1, @output_file_name = N'', @on_success_step_id = 0, @on_success_action = 1, @on_fail_step_id = 0, @on_fail_action = 2
  EXEC sp_update_job @job_id = @JobID, @start_step_id = 1 
  EXEC sp_add_jobschedule @job_id = @JobID, @name = N'CA7 autodiscovery load', @enabled = 1, @freq_type = 4, @active_start_date = 20010118, @active_start_time = 233000, @freq_interval = 1, @freq_subday_type = 1, @freq_subday_interval = 0, @freq_relative_interval = 0, @freq_recurrence_factor = 1, @active_end_date = 99991231, @active_end_time = 235959
  EXEC sp_add_jobserver @job_id = @JobID, @server_name = N'(local)', @automatic_post = 0 
END
GO

-- PRINT 'Creating scheduled task "CleanUp CA7 old jobs"'
-- GO
-- 
-- IF NOT EXISTS (SELECT * FROM sysjobs WHERE name = 'CleanUp CA7 old jobs')
-- BEGIN 
--   DECLARE @JobID BINARY(16)  
--   EXECUTE sp_add_job @job_id = @JobID OUTPUT , @job_name = N'CleanUp CA7 old jobs', @enabled = 1, @owner_login_name = N'sa', @description = N'', @category_name = N'[Uncategorized (Local)]', @notify_level_eventlog = 2, @delete_level= 0
--   EXECUTE sp_add_jobstep @job_id = @JobID, @step_id = 1, @step_name = N'CleanUp', @command = N'asisp_cleanupCA7  7, 30', @database_name = N'Object', @server = N'', @database_user_name = N'', @subsystem = N'TSQL', @cmdexec_success_code = 0, @flags = 0, @retry_attempts = 0, @retry_interval = 1, @output_file_name = N'', @on_success_step_id = 0, @on_success_action = 1, @on_fail_step_id = 0, @on_fail_action = 2
--   EXECUTE sp_update_job @job_id = @JobID, @start_step_id = 1 
--   EXECUTE sp_add_jobschedule @job_id = @JobID, @name = N'CleanUp', @enabled = 1, @freq_type = 4, @active_start_date = 20010301, @active_start_time = 230000, @freq_interval = 1, @freq_subday_type = 1, @freq_subday_interval = 0, @freq_relative_interval = 0, @freq_recurrence_factor = 1, @active_end_date = 99991231, @active_end_time = 235959
--   EXECUTE sp_add_jobserver @job_id = @JobID, @server_name = N'(local)', @automatic_post = 0 
-- END
-- GO 

-------------------------------------------------------------------------------------------------------------
USE Object 
GO

-- Create default global variable for ca7 keep_unknown
IF NOT EXISTS (SELECT * FROM SystemConfiguration WHERE groupname = 'CA7' AND propertyname = 'keep_unknown')
  EXEC asisp_setCA7Globals 'CA7', 'keep_unknown', '0', 'TINYINT' 
GO

-- setup CA7 bcys name lookup table
-- EXEC asisp_setCA7bcysnamelookup 'short_name', 'full_name' 
