PRINT 'RECREATING PROCEDURE _GetBatchCycleJobHist....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_GetBatchCycleJobHist' AND type = 'P')
  DROP PROCEDURE _GetBatchCycleJobHist
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/UpgradeBatch.sqv  $
-- $Revision:   1.6  $
-- $Date:   Sep 06 2000 11:11:12  $
-- $Author:   BRENT  $
CREATE PROCEDURE _GetBatchCycleJobHist
  @cid              ClassID      = null,
  @id               ObjID        = null,
  @keyJobIDExpr     VARCHAR(255) = null,
  @nonkeyJobIDExpr  VARCHAR(255) = null,
  @alertStateExpr   VARCHAR(255) = null,
  @currentStateExpr VARCHAR(255) = null,
  @desiredStateExpr VARCHAR(255) = null,
  @minTime          DATETIME     = null,
  @maxTime          DATETIME     = null,
  @strict           TINYINT      = 0,
  @jobTypeMask      INT          = 3      -- Bit values: 1=non-key, 2=key
as
  -- Do not allow @maxTime to be specified without @minTime (we don't want to SELECT back to the BEGINning of time!)
  IF @maxTime IS NOT NULL AND @minTime IS NULL
  BEGIN
    RAISERROR('_GetBatchCycleJobHist: Cannot specify a max time without a min time', 11, 1)
    RETURN -30000
  END

  -- Assemble the portion of the where-clause having to do with alert state
  DECLARE @alertStateClause VARCHAR(255)
  IF @alertStateExpr IS NOT NULL
    SELECT @alertStateClause = ' AND alertstate' + @alertStateExpr

  -- Assemble the portion of the where-clause having to do with current state
  DECLARE @currentStateClause VARCHAR(255)
  IF @currentStateExpr IS NOT NULL
    SELECT @currentStateClause = ' AND currentstate' + @currentStateExpr

  -- Assemble the portion of the where-clause having to do with desired state
  DECLARE @desiredStateClause VARCHAR(255)
  IF @desiredStateExpr IS NOT NULL
    SELECT @desiredStateClause = ' AND desiredstate' + @desiredStateExpr

  -- Assemble the variable declarations for dynamic SQL
  DECLARE @cidDecl VARCHAR(255)
  SELECT @cidDecl = 'DECLARE @cid ClassID SELECT @cid='
  IF @cid IS NULL
    SELECT @cidDecl = @cidDecl + 'NULL '
  ELSE
    SELECT @cidDecl = @cidDecl + '''' + rtrim(@cid) + ''' '

  DECLARE @now DATETIME
  SELECT @now = GETDATE()
  IF @minTime IS NULL
    SELECT @minTime = DATEADD(mm,-1,@now)
  IF @maxTime IS NULL
    SELECT @maxTime = @now

  DECLARE @minTimeDecl VARCHAR(255)
  SELECT @minTimeDecl = 'DECLARE @minTime datetime SELECT @minTime=''' + CONVERT(VARCHAR(40),@minTime,113) + ''' '

  DECLARE @maxTimeDecl VARCHAR(255)
  SELECT @maxTimeDecl = 'DECLARE @maxTime datetime SELECT @maxTime=''' + CONVERT(VARCHAR(40),@maxTime,113) + ''' '

  DECLARE @strictDecl VARCHAR(255)
  SELECT @strictDecl = 'DECLARE @strict tinyint SELECT @strict=' + CONVERT(VARCHAR(3), @strict)

  DECLARE @keyJobSelect VARCHAR(255)
  IF @keyJobIDExpr IS NOT NULL
    SELECT @keyJobSelect = 'INSERT #bcj SELECT b.cid, b.id, b.name, b.id, b._ParentCID, b._ParentID ' +
                           'FROM BATCH_V AS b ' +
                           'WHERE b._KeyStatus = 1 AND b.id ' + @keyJobIDExpr

  DECLARE @nonkeyJobSelect VARCHAR(255)
  IF @nonkeyJobIDExpr IS NOT NULL
    SELECT @nonkeyJobSelect = 'INSERT #bcj SELECT b.cid, b.id, b.name, b.id, b._ParentCID, b._ParentID ' +
                              'FROM BATCH_V AS b ' + 
                              'WHERE b._KeyStatus = 0 AND id ' + @nonkeyJobIDExpr +
                              ' AND b.id NOT IN (SELECT jobid FROM #bcj)'

  -- If both @keyJobSelect and @nonkeyJobSelect are null then we use the calling object's id
  IF @keyJobSelect IS NULL AND @nonkeyJobSelect IS NULL
  BEGIN
    IF @cid = 'BATC'
      SELECT @keyJobSelect = 'INSERT #bcj SELECT b.cid, b.id, b.name, b.id, b._ParentCID, b._ParentID ' +
                             'FROM BATCH_V AS b ' + 
                             'WHERE b.id =' + CONVERT(VARCHAR(9), @id)
    ELSE IF @cid = 'BTCY'
    BEGIN
      SELECT @keyJobSelect = 
        'INSERT #bcj SELECT b.cid, b.id, b.name, b.id, b._ParentCID, b._ParentID ' +
        'FROM BATCH_V AS b ' + 
        'WHERE b._KeyStatus = 1 AND b._ParentCID = ''BTCY'' AND b._ParentID = ' + CONVERT(VARCHAR(9), @id)
      SELECT @nonkeyJobSelect = 
        'INSERT #bcj SELECT b.cid, b.id, b.name, b.id, b._ParentCID, b._ParentID ' +
        'FROM BATCH_V AS b ' + 
        'WHERE b._KeyStatus = 0 AND b._ParentCID = ''BTCY'' AND b._ParentID = ' + CONVERT(VARCHAR(9), @id)
    END
    ELSE IF @cid = 'BCYS'
    BEGIN
      SELECT @keyJobSelect = 
        'INSERT #bcj SELECT b.cid, b.id, b.name, b.id, b._ParentCID, b._ParentID ' +
        'FROM BATCH_V AS b ' + 
        '  JOIN BCYS_BTCY_AGGC AS link ON b._ParentCID = ''BTCY'' AND b._ParentID = link.dst_id ' +
        'WHERE b._KeyStatus = 1 AND link.src_id = ' + CONVERT(VARCHAR(9), @id)
      SELECT @nonkeyJobSelect =
        'INSERT #bcj SELECT b.cid, b.id, b.name, b.id, b._ParentCID, b._ParentID ' +
        'FROM BATCH_V AS b ' + 
        '  JOIN BCYS_BTCY_AGGC AS link ON b._ParentCID = ''BTCY'' AND b._ParentID = link.dst_id ' +
        'WHERE b._KeyStatus = 0 AND link.src_id = ' + CONVERT(VARCHAR(9), @id)
    END
    ELSE
    BEGIN
      RAISERROR('Attempt to call GetBatchCycleJobHist for unsupported object class %s', 11, 1, @cid)
      RETURN -30000
    END
  END

  DECLARE @jobTypeMaskExp VARCHAR(255)
  IF (@jobTypeMask & 3) = 3
    SELECT @jobTypeMaskExp = ''
  ELSE
    SELECT @jobTypeMaskExp = 'AND (_KeyStatus & ' + CONVERT(VARCHAR(1), @jobTypeMask) + ' = ' + CONVERT(VARCHAR(1), @jobTypeMask)

  RAISERROR('@alertStateExpr:%s', 0, 1, @alertStateExpr)
  RAISERROR('@keyJobSelect:%s', 0, 1, @keyJobSelect)
  RAISERROR('@nonkeyJobSelect:%s', 0, 1, @nonkeyJobSelect)
  RAISERROR('@jobTypeMaskExp:%s', 0, 1, @jobTypeMaskExp)

  EXEC(
  @minTimeDecl +
  @maxTimeDecl +
  @strictDecl +
  "
  -- Step 1 - collect ids of all jobs into temp table
  CREATE TABLE #bcj
  (
    obj_cid             ClassID,        --   4
    obj_id              INT null,       --   5
    name                ObjName,        --  54
    jobid               INT,            --   4
    parent_cid          ClassID,        --   4
    parent_id           ObjID           --   4
  ) " +
  @keyJobSelect + " " +
  @nonkeyJobSelect + "

  DECLARE @rc INT

  -- Create a temp table to hold the state history for the #bcj rows that occurred within the specified time range
  CREATE TABLE #hist
  (
    id           INT,
    jobid        INT PRIMARY KEY (jobid,time),
    time         DATETIME,
    currentstate INT,
    desiredstate INT,
    eventcid     ClassID NULL,
    eventid      INT,
    eventname    VARCHAR(16) NULL,
    srctime      DATETIME NULL,
    alertstate   INT,
    priority     INT,
    seqno        VARCHAR(16) NULL,
    detail       VARCHAR(255) NULL
  )
  IF @maxTime IS NULL
    INSERT #hist SELECT h.id, h.jobid, h.time, h.currentstate, h.desiredstate, h.eventcid, h.eventid, null, null, 0, 0, null, null
    FROM BatchCycleJobHist AS h JOIN #bcj AS j ON j.jobid = h.jobid
    WHERE h.time >= @minTime
  ELSE
    INSERT #hist SELECT h.id, h.jobid, h.time, h.currentstate, h.desiredstate, h.eventcid, h.eventid, null, null, 0, 0, null, null
    FROM BatchCycleJobHist AS h JOIN #bcj AS j ON j.jobid = h.jobid
    WHERE h.time >= @minTime and h.time <= @maxTime
  SELECT @rc = @@rowcount

  -- Also must select the beginning state for the time range
  DECLARE @rc2 INT
  INSERT #hist SELECT h.id, h.jobid, @minTime, h.currentstate, h.desiredstate, h.eventcid, h.eventid, null, null, 0, 0, null, null
  FROM BatchCycleJobHist AS h JOIN #bcj AS j ON j.jobid = h.jobid
  WHERE 
    h.time = (SELECT MAX(hh.time) FROM BatchCycleJobHist AS hh WHERE hh.jobid = h.jobid AND hh.time < @minTime)
    AND NOT EXISTS (SELECT * FROM #hist WHERE #hist.jobid = h.jobid and #hist.time = @minTime)
  SELECT @rc2 = @@rowcount

  -- Update event related details
  UPDATE h SET 
    alertstate = _AlertState,
    priority = _Priority,
    eventname = _MESSAGE,
    srctime = _TIME,
    seqno = _SEQ_NO,
    detail = _REASON_CD 
  FROM #hist AS h JOIN Message_A ON h.eventcid = 'MESG' AND Message_A.id = h.eventid

  IF @strict = 0 -- non-strict match
  BEGIN
    IF @maxTime IS NULL
      SELECT
        cid='BCJH',
        id,
        name,
        parent_cid,
        parent_id,
        obj_cid,
        obj_id,
        currentStateID=currentstate,
        desiredStateID=desiredstate,
        eventCID=eventcid,
        eventID=eventid,
        eventName=eventname,
        srctime,
        ctime=time,
        alertStateID=alertstate,
        priorityID=priority,
        seqno,
        eventDetail=detail
      FROM
        #bcj AS j,
        #hist AS h
      WHERE
        h.time >= @minTime AND h.jobid = j.jobid " +
        @alertStateClause +
        @currentStateClause +
        @desiredStateClause + 
        @jobTypeMaskExp + "
    ELSE  -- maxTime specified
      SELECT
        cid='BCJH',
        id,
        name,
        parent_cid,
        parent_id,
        obj_cid,
        obj_id,
        currentStateID=currentstate,
        desiredStateID=desiredstate,
        eventCID=eventcid,
        eventID=eventid,
        eventName=eventname,
        srctime,
        ctime=time,
        alertStateID=alertstate,
        priorityID=priority,
        seqno,
        eventDetail=detail
      FROM
        #bcj j,
        #hist h
      WHERE
        h.time >= @minTime and h.time <= @maxTime and h.jobid = j.jobid " +
        @alertStateClause +
        @currentStateClause +
        @desiredStateClause + 
        @jobTypeMaskExp + "
  END
  -- If @strict is true then we are looking only for rows with one state record that matches the specified current and/or desired state
  ")

