PRINT 'RECREATE PROCEDURE _PollForCompletedSchedules....'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_PollForCompletedSchedules' AND type = 'P')
  DROP PROCEDURE _PollForCompletedSchedules
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/ISSUE_01603_20001026.sqv  $
-- $Revision:   1.0  $
-- $Date:   Oct 26 2000 10:41:36  $
-- $Author:   BRENT  $
-- Checks for batch schedules that contain only completed jobs that are not themselves
-- in a completed state. Generates a 'OPCENDED' message on such schedules and directly
-- sets their state to 'Completed'.
-- This procedure is installed as a scheduled task that runs periodically throughout
-- the day.
CREATE PROCEDURE _PollForCompletedSchedules
  @cid ClassID,
  @id  ObjID
AS
  SELECT DISTINCT s.id AS id, s._CurrentStateID AS sched_state, b._CurrentStateID AS job_state
  INTO #btcy_job_state
  FROM BATCH_V AS b JOIN BatchCycle_V AS s ON s.id = b._ParentID
  WHERE b._ParentCID = 'BTCY'

  -- delete btcys where its children have job_state other then 83 and 0 
  DELETE b1 FROM 
    #btcy_job_state AS b1
    JOIN #btcy_job_state AS b2 ON b2.id = b1.id
    WHERE b1.sched_state <> 93 
    AND (b1.job_state = 83 AND b2.job_state NOT IN (83,0))

  -- delete btcys of which chilren have job_state other then 83 
  DELETE #btcy_job_state 
    WHERE sched_state = 93 OR job_state <> 83

  IF EXISTS (SELECT * FROM #btcy_job_state)
  BEGIN
    INSERT StagedMESG(parent_cid, parent_id, TIME, SEQ_NO, OBJECT_NAME, NATIVE_KEY, DISCOVERY_IND, STATE, MESSAGE, REASON_CD)
      SELECT 'BTCY', b.id, GETDATE(), '', s.name, '', 'MON', 'COMPLETED', 'OPCENDED', 'ALL OPERATIONS IN APPLICATION ' + s.name + ' COMPLETED AS OF ' + CONVERT(VARCHAR(40), GETDATE(), 120)
      FROM #btcy_job_state AS b JOIN BatchCycle_V AS s ON s.id = b.id
    UPDATE btcy SET _CurrentStateID = 93 FROM BatchCycle_V AS btcy JOIN #btcy_job_state AS js ON js.id = btcy.id
  END

